local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")
local module = mHud:NewModule("Abyss")

mHud.defaults.profile.abyss = {
	fps = 60,
}

mHud.defaults.profile.bar['*'].abyss = { 
	active = false,
	size = 1.0,
	hold = 7,
	color = { r = 1, g = 0, b = 0, a = 1 },
	
	gradientColor = "Abyss",
	gradientAlpha = "Abyss",
	gradientColorCombat = nil,
	gradientAlphaCombat = nil,
	
	text = {
		active = false,
		
		size = 16,
		
		offsetX = 15,
		offsetY = 0,
		
		font = "Arial Narrow",
		gradientAlpha = "AbyssText",
		gradientAlphaCombat = "AbyssText",
		
		color = {
			r = 1,
			g = 0,
			b = 0,
			a = 1
		},
		
		shadow = {
			r = 0,
			g = 0,
			b = 0,
			a = 1,
			offsetX = -1,
			offsetY = 0,
		},
		
		anchorToSpark = true,
		
		anchor = {
			point = "CENTER",
			to = "UIParent",
			relative = "CENTER",
		},
	},
}

module.frames = { }
module.lastRun = 0

module.colorTemplates = {
	Normal = {
		Abyss = {
			[1] = { v = 1.0, c = { r = 1.00, g = 0.00, b = 0.00 }},
			[2] = { v = 0.0, c = { r = 1.00, g = 1.00, b = 1.00 }},
		},
	},
}

module.alphaTemplates = {
	Normal = {
		Abyss = {
			[1] = { v = 1.00, c = { a = 0.00, b = "BLEND" }},
			[2] = { v = 0.90, c = { a = 0.80, b = "BLEND" }},
			[3] = { v = 0.00, c = { a = 1.00, b = "BLEND" }},
		},
		AbyssText = {
			[1] = { v = 1.00, c = { a = 0.00, b = "BLEND" }},
		},
	},
	Combat = {
		AbyssText = {
			[1] = { v = 1.00, c = { a = 0.00, b = "BLEND" }},
			[2] = { v = 0.90, c = { a = 0.00, b = "BLEND" }},
			[3] = { v = 0.50, c = { a = 1.00, b = "BLEND" }},
			[4] = { v = 0.00, c = { a = 1.00, b = "BLEND" }},
		},
	},
}



function module:OnEnable()
	--self:Debug("Loaded", true)
	if not self.event then self.event = CreateFrame("Frame") end
	self:RegisterEvents()
end

function module:OnDisable()
	self:UnregisterEvents()
end

function module:KeepAlive()
	if not self.runing then
		self.event:SetScript("OnUpdate", function() self:OnUpdate() end)
		self.runing = true
	end
end

function module:OnUpdate()

	if self.lastRun + 1 / mHud.db.profile.abyss.fps > GetTime() then return end
	self.lastRun = GetTime()
	
	self.any = false

	for _,f in pairs(self.frames) do
	
		if f.abyss.active == false then
		elseif (f.value == 1 or f.value <= 0.001) and f.abyss.holdTo < GetTime() then
		
			f.abyss.active = false
			f.abyss:Hide()
			if f.abyss.text then
				if f.abyss.text:GetAlpha() > .2 then
					UIFrameFadeOut(f.abyss.text, f.db.abyss.hold, 1, 0)
				else
					f.abyss.text:Hide()
				end
			end
			--self:Debug("End: "..f.id, true)
			
		else
		
			self.any = true
	
			if f.abyss.holdTo < GetTime() or f.abyss.oldValue ~= f.abyss.value then

				if f.abyss.holdTo < GetTime() and f.abyss.value < f.value then
					f.abyss.value = f.abyss.value + (f.value - f.abyss.value) / 20 + 0.001
				else
					--self:Debug("Wait: "..(f.abyss.holdTo-GetTime()).." Hold: "..f.value, true)
				end
				--self:Debug("Update: "..f.id, true)
				
				if f.rotation == 0 or f.rotation == 2 then
					f.abyss.low = f.abyss.value - f.db.abyss.size / f:GetWidth()
					f.abyss.high = f.abyss.value + f.db.abyss.size / f:GetWidth()
				else
					f.abyss.low = f.abyss.value - f.db.abyss.size / f:GetHeight()
					f.abyss.high = f.abyss.value + f.db.abyss.size / f:GetHeight()
				end
				
				if f.abyss.low < 0 then f.abyss.low = 0 end
				if f.abyss.high > 1 then f.abyss.high = 1 end
				
				mBar:SetSubValue(f.abyss, f.abyss.low, f.abyss.high)
				mBar:PaintTexture(f.abyss, f.abyss.low, f.abyss.high)
				
				if f.db.abyss.gradientColor and mHud:GetModule("GradientColor", true) then
					self.rTab = mHud:GetModule("GradientColor"):GetProfile(f, f.db.abyss.gradientColor, f.db.abyss.gradientColorCombat, f.abyss.value)
					f.abyss:SetVertexColor(self.rTab.r, self.rTab.g, self.rTab.b)
				end
				if f.db.abyss.gradientAlpha and mHud:GetModule("GradientAlpha", true) then
					self.rTab = mHud:GetModule("GradientAlpha"):GetProfile(f, f.db.abyss.gradientAlpha, f.db.abyss.gradientAlphaCombat, f.abyss.value)
					f.abyss:SetAlpha(self.rTab.a)
				end
				
				if f.db.abyss.text.active then
				
					f.abyss.text:SetText(math.floor(f.abyss.value * f.handler:GetUnitMax(f.db.unit)))
					 
					
					if f.db.abyss.gradientAlpha and mHud:GetModule("GradientAlpha", true) then
						self.rTab = mHud:GetModule("GradientAlpha"):GetProfile(f, f.db.abyss.text.gradientAlpha, f.db.abyss.text.gradientAlphaCombat, f.abyss.value)
						f.abyss.text:SetAlpha(self.rTab.a)
					else
						f.abyss.text:SetAlpha(1)
					end
					
					--self:Debug(f.alpha.a, true)
					
				end
				
				f.abyss.oldValue = f.abyss.value
				
			end
		end
	end
	
	if not self.any then
		self.event:SetScript("OnUpdate", nil)
		self.runing = false
	end
	
end

function module:RegisterEvents()

	self:RegisterMessage("MBAR_CONFIG_UPDATE", "BarConfigChange")
	self:RegisterMessage("MBAR_ANCHOR_SET", "OnAnchor")
	self:RegisterMessage("MBAR_PAINT", "OnPaint")
	
end

function module:UnregisterEvents()

	self:UnregisterMessage("MBAR_CONFIG_UPDATE")
	self:UnregisterMessage("MBAR_ANCHOR_SET")
	self:UnregisterMessage("MBAR_PAINT")
	
end

function module:BarConfigChange(_, f)

	if f.db.abyss.active then
		if not f.abyss then f.abyss = f:CreateTexture(nil, "OVERLAY") end
		f.abyss.active = false
		f.abyss.value = 1
		f.abyss.oldValue = 1
		f.abyss.holdTo = 0
		f.abyss:SetHeight(f:GetHeight())
		f.abyss:SetWidth(f:GetWidth())
		f.abyss:SetTexture(f.bar:GetTexture())
		f.abyss:SetVertexColor(f.db.abyss.color.r, f.db.abyss.color.g, f.db.abyss.color.b, f.db.abyss.color.a)
		self.frames[f.id] = f
		
		if f.db.abyss.text.active then
			if not f.abyss.text then f.abyss.text = f:CreateFontString(nil, "OVERLAY") end
			local t = f.abyss.text
			
			t:Show()
			t:SetTextHeight(f.db.abyss.text.size)
			t:SetFont(mBar.media:Fetch("font", f.db.abyss.text.font), f.db.abyss.text.size)
			
			t:SetVertexColor( f.db.abyss.text.color.r, f.db.abyss.text.color.g, f.db.abyss.text.color.b, f.bar:GetAlpha() )
			
			t:SetShadowColor( f.db.abyss.text.shadow.r, f.db.abyss.text.shadow.g, f.db.abyss.text.shadow.b, f.db.abyss.text.shadow.a )									
			t:SetShadowOffset( f.db.abyss.text.shadow.offsetX, f.db.abyss.text.shadow.offsetY )
			
			t:ClearAllPoints()
			if (f.db.abyss.text.anchorToSpark) then
				t:SetPoint(
					-- Point
					f.db.abyss.text.anchor.point or "CENTER",
					-- To
					f.abyss,
					-- Relative
					f.db.abyss.text.anchor.relative or "CENTER",
					-- X
					f.db.abyss.text.offsetX,
					-- Y
					f.db.abyss.text.offsetY)
			end
		else
			if f.abyss.text then f.abyss.text:Hide() end
		end
		
	else
		if f.abyss then
			f.abyss:Hide()
			self.frames[f.id] = nil
		end
	end

end

function module:OnAnchor(_, f)
	if f.db.abyss.text.active and f.abyss and not f.db.abyss.text.anchorToSpark then
		local t = f.abyss.text
		t:ClearAllPoints()
		t:SetPoint(
			-- Point
			f.db.abyss.text.anchor.point or "CENTER",
			-- To
			(type(f.db.abyss.text.anchor.to) == "number" and mBar:GetFrame(f.db.abyss.text.anchor.to))
			or f.db.abyss.text.anchor.to
			or "UIParent",
			-- Relative
			f.db.abyss.text.anchor.relative or "CENTER",
			-- X
			f.db.abyss.text.offsetX,
			-- Y
			f.db.abyss.text.offsetY)
	end
end

function module:OnPaint(_, f)

	if f.db.abyss.active then
		if f.abyss.value > f.value and f:IsShown() and f.value > 0.001 then
			f.abyss.value = f.value
			f.abyss.holdTo = GetTime() + f.db.abyss.hold
			f.abyss.active = true
			--self:Debug("Start: "..f.id, true)
			f.abyss:Show()
			if f.abyss.text then f.abyss.text:Show() end
			self:KeepAlive()
		end
		
	end
	
end





