local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")
local L = LibStub("AceLocale-3.0"):GetLocale("mHud:Configuration")--, true
local AceDBOptions = LibStub("AceDBOptions-3.0")
local module = mHud:NewModule("Configuration")
local line = "------------------------------------"

mHud.defaults.profile.configuration = {
	help = true,
	fps = 8,
	step = 0.03,
}

module.editValues = {
	last = 0,
	dir = 1,
	value = 0.5,
}

module.drag = false
module.edit = false
module.combat = false

module.modConfig = { }
module.tmp = { }
module.copy = { }

module.rotation = {
	[0] = L['Rotation0'],
	[1] = L['Rotation90'],
	[2] = L['Rotation180'],
	[3] = L['Rotation270'],
}

module.textureRot = {
	[0] = L['Rotation0'],
	[1] = L['Rotation90'],
	[2] = L['Rotation180'],
	[3] = L['Rotation270'],
}

module.relativeList = {
	TOPLEFT		= L['TOPLEFT'],
	TOP			= L['TOP'],
	TOPRIGHT	= L['TOPRIGHT'],
	LEFT		= L['LEFT'],
	CENTER		= L['CENTER'],
	RIGHT		= L['RIGHT'],
	BOTTOMLEFT	= L['BOTTOMLEFT'],
	BOTTOM		= L['BOTTOM'],
	BOTTOMRIGHT	= L['BOTTOMRIGHT'],
}


module.options = {
	type = "group",
	name = "mHud",
	childGroups = "tree",
	plugins = {},
	args = {
		desc = {
			type = "execute",
			name = L['Load menu'],
			func = function() module:WriteConfig() end,
		},
	},
}

function module:OnEnable()

	if not self.event then self.event = CreateFrame("Frame") end

	if not self.globalsSet then
		-- Config is created when /mhud is run
		LibStub("AceConfig-3.0"):RegisterOptionsTable("mHud", module.options, "mHudConfig")
		LibStub("AceConsole-3.0"):RegisterChatCommand("mhud", function() self:WriteConfig(); LibStub("AceConfigDialog-3.0"):Open("mHud") end)
		self.globalsSet = true
	end
	
end


function module:SetEdit(value)
	self.edit = value
	module:SendMessage("MHUD_CONFIG_EDIT", value)
	if value == false then
		if self.combat then self:SetCombat(false, true) end
		if self.drag then self:SetDrag(false, true) end
		self:LeaveEditMode()
	else
		self:EnterEditMode()
		if self.combat then self:SetCombat(self.combat, true) end
		if self.drag then self:SetDrag(self.drag, true) end
	end
end


function module:SetDrag(value, skipSave)
	if not skipSave then self.drag = value end
	
	if value then
		self:EnterDragMode()
	else
		self:LeaveDragMode()
	end
	
	module:SendMessage("MHUD_CONFIG_DRAG", value)
end


function module:SetCombat(value, skipSave)

	if not skipSave then self.combat = value end
	if mHud.mTemplate then mHud.mTemplate.inCombat = value end
	module:SendMessage("MHUD_CONFIG_COMBAT", value)
	
end


function module:UpdateConfig(id)
	self:Debug("Updating config: "..id)
	mBar:ConfigChange(mBar:GetFrame(id))
end

function module:BarIdList(i)
	local t = {}
	for id,db in pairs(mHud.db.profile.bar) do
		if i ~= id then
			t[id] = L['Bar']..id..' '..db.unit..' '..db.type
		end
	end
	return t
end

function module:BarList(i)
	local t = {}
	for id,db in pairs(mHud.db.profile.bar) do
		if i ~= id then
			t["NUMBER:"..id] = L['Bar']..id..' '..db.unit..' '..db.type
		end
	end
	return t
end

function module:GetTextures()
	local t = {}
	for k,v in pairs(mBar.media:List("statusbar")) do t[v] = v end
	return t
end

function module:WriteConfig()

	local dbOpt = AceDBOptions:GetOptionsTable(mHud.db)
	dbOpt.handler.Reset = module.ResetProfile
	dbOpt.handler.SetProfile = module.SetProfile
	
	self.options.args = {
		start = {
			type = "group",
			name = L['Home'],
			order = 10,
			childGroups = "tab",
			args = {
				welcome = {
					type = "group",
					name = L['Welcome'],
					order = 10,
					childGroups = "tree",
					args = {
						sdesc1 = {
							order = 10,
							type = "description",
							name = L['Welcome to mHud'],
						},
						sdesc1line = { order = 11, type = "description", name = line },
						sdesc3 = {
							order = 20,
							type = "description",
							name = L["Welcome Desc1"],
						},
						sdesc10space = { order = 29, type = "description", name = " " },
						sdesc10 = {
							order = 30,
							type = "description",
							name = L["Welcome Desc2"],
						},
						sdesc20space = { order = 49, type = "description", name = " " },
						sdesc20 = {
							order = 50,
							type = "description",
							name = L["Help out?"],
						},
						sdesc20line = { order = 51, type = "description", name = line },
						sdesc21 = {
							order = 52,
							type = "description",
							name = L["Help out Desc1"],
						},
					},
				},
				global = {
					type = "group",
					name = L["Global settings"],
					order = 20,
					childGroups = "tree",
					args = {
						desc30 = {
							order = 90,
							type = "description",
							name = L["Display Help Desc"],
						},
						config = {
							order = 91,
							type = "toggle",
							name = L["Display Help"],
							get = function() return mHud.db.profile.configuration.help end,
							set = function(info, value) mHud.db.profile.configuration.help = value; self:WriteConfig() end,
						},
					},
				},
				edit = {
					type = "group",
					name = L["Edit mode"],
					order = 30,
					childGroups = "tree",
					args = {
						desceditheader = {
							order = 5,
							type = "description",
							name = L["Edit mode"],
						},
						desceditline = { order = 6, type = "description", name = line },
						descedit1 = {
							order = 9,
							type = "description",
							name = L["Edit mode Desc"],
						},
						editmode = {
							order = 10,
							type = "toggle",
							name = L["Edit mode"],
							get = function() return module.edit end,
							set = function(info, value) self:SetEdit(value) end,
						},
						desccombat1 = {
							order = 29,
							type = "description",
							name = L["Combat mode Desc"],
						},
						combat = {
							order = 30,
							type = "toggle",
							name = L["Combat mode"],
							get = function() return module.combat end,
							set = function(info, value) self:SetCombat(value) end,
							disabled = function() return not module.edit end,
						},
						descdragheaderspace = { order = 49, type = "description", name = " " },
						descdragheader = {
							order = 50,
							type = "description",
							name = L["Drag Mode"],
						},
						descdragline = { order = 51, type = "description", name = line },
						descdrag1 = {
							order = 61,
							type = "description",
							name = L["Drag Mode Desc"],
						},
						desc1space = { order = 62, type = "description", name = " " },
						descdrag2 = {
							order = 65,
							type = "description",
							name = L["Drag Mode Desc2"],
						},
						desc2space = { order = 66, type = "description", name = " " },
						descdrag3 = {
							order = 70,
							type = "description",
							name = L["Drag Mode Desc3"],
						},
						drag = {
							order = 80,
							type = "toggle",
							name = L["Drag Mode"],
							get = function() return module.drag end,
							set = function(info, value) self:SetDrag(value) end,
							disabled = function() return not module.edit end,
						},
					},
				},
			},
		},
		bar = {
			type = "group",
			name = L["Bars"],
			order = 20,
			childGroups = "tree",
			args = self:BarMenu(),
		},
		skin = {
			type = "group",
			name = L["Skins"],
			order = 30,
			childGroups = "tree",
			args = self:SkinMenu(),
		},
		module = {
			type = "group",
			name = L["Modules"],
			order = 50,
			childGroups = "tree",
			args = self:ModuleMenu(),
		},
		handler = {
			type = "group",
			name = L["Handlers"],
			order = 60,
			childGroups = "tree",
			args = self:HandlerMenu(),
		},
		profile = dbOpt,
		--[[{
			type = "group",
			name = L["Profiles"],
			order = 100,
			childGroups = "tree",
			plugins = {},
			args = 
			{
				reset = {
					order = 1,
					type = "execute",
					name = L["Reset Profile"],
					desc = L["Reset Profile Desc"],
					func = function() self:ResetProfile() end,
				},
			},
		},--]]
	}
	
	if mHud.db.profile.configuration.help ~= true then self:RemoveDesc(self.options.args) end
	
end


function module:RemoveDesc(tbl, pk)

	if pk == "args" then
		for k,v in pairs(tbl) do
			if string.sub(k, 1, 4) == "desc" then
				tbl[k] = nil
			else
				if type(v) == "table" then self:RemoveDesc(tbl[k], k) end
			end
		end
	else
		for k,v in pairs(tbl) do
			if type(v) == "table" then self:RemoveDesc(tbl[k], k) end
		end
	end

end

function module:NewBar()

	local id = # mHud.db.profile.bar + 1
	mHud.db.profile.bar[id].active = true
	mHud.bars[id] = mBar:CreateBar(id, mHud.db.profile.bar[id])
	
	self:WriteConfig()
	
end

function module:ListTypes()
	local t = {}
	for name, module in mBar:IterateModules() do
		if name ~= "None" then t[name] = module.description or name end
	end
	return t
end

function module:AnchorList(id)
	local t = module:BarList(id)
	t['UIParent'] = "UIParent"
	return t
end

function module:AnchorGet(from)
	if type(from) == "number" then
		return "NUMBER:"..from
	else
		return from
	end
end

function module:AnchorSet(value, db)
	if string.sub(value, 1, 7) == "NUMBER:" then
		value = tonumber(string.sub(value, 8))
	end
	db.anchor.to = value
end


function module:BarMenu()

	local t = {
		desc1 = {
			order = 10,
			type = "description",
			name = L["Bar configuration"],
		},
		desc1line = { order = 11, type = "description", name = line },
		desc2 = {
			order = 20,
			type = "description",
			name = L["Bar configuration Desc1"],
		},
		desc4space = { order = 24, type = "description", name = " " },
		desc4 = {
			order = 25,
			type = "description",
			name = L["Bar configuration Desc2"],
		},
		desc6space = { order = 29, type = "description", name = " " },
		desc6 = {
			order = 30,
			type = "description",
			name = L["Bar configuration Desc3"],
		},
		desc7space = { order = 58, type = "description", name = " " },
		desc7 = {
			order = 59,
			type = "description",
			name = L["Bar new Desc"],
		},
		add = {
			order = 60,
			type = "execute",
			name = L["Bar new"],
			func = function() self:NewBar()end,
		},
		bulk = {
			type = "group",
			order = 200,
			name = L["Bulk copy"],
			childGroups = "tab",
			plugins = {},
			args = {
				from = {
					order = 10,
					type = "select",
					name = L["Copy from"],
					values = self:BarIdList(),
					get = function() return self.copy.from end,
					set = function(info, value) self.copy.from = value; self:WriteConfig() end,
				}
			}
		},
	}
	
	for id,db in pairs(mHud.db.profile.bar) do

		if db.active and mBar:GetModule(db.type, true) then 

			local name = L['Bar']..id..' '..db.unit..' '..db.type
			
			t['bar'..id] = {
				type = "group",
				order = id,
				name = name,
				childGroups = "tree",
	 			plugins = {},
				args = {
					unit = {
						order = 2,
						type = "input",
						name = L["Unit"],
						get = function() return db.unit end,
						set = function(info, value) db.unit = value; self:ReloadmHud(); end,
					},
					type = {
						order = 3,
						type = "select",
						name = L["Type"],
						get = function() return db.type end,
						set = function(info, value) db.type = value; self:ReloadmHud(); end,
						values = self:ListTypes(),
						style = "dropdown",
					},
					delete = {
						order = 99,
						type = "execute",
						name = L["Remove Bar"],
						func = function()
							mHud.db.profile.bar[ id ] = { } -- Overwrite with dummy table
							mHud.db.profile.bar[ id ].active = false -- not active
							self:ReloadmHud();
						end,
						confirm = true,
						confirmText = L["Remove Bar Confirm"],
					},
					desc1 = {
						type = "description",
						order = 100,
						name = L["Bar Desc1"],
					},
					desc1space = {
						type = "description",
						order = 101,
						name = " ",
					},
					desc2 = {
						order = 102,
						type = "description",
						name = L["Bar Desc2"],
					},
					size = {
						type = "group",
						order = 10,
						name = L["Size & Position"],
						childGroups = "tab",
						plugins = {},
						args = {
							width = {
								order = 10,
								type = "range",
								name = L["Width"],
								get = function() return db.width end,
								set = function(info, value) db.width = value; self:UpdateConfig(id) end,
								min = 1, max = 600, bigStep = 10,
							},
							height = {
								order = 11,
								type = "range",
								name = L["Height"],
								get = function() return db.height end,
								set = function(info, value) db.height = value; self:UpdateConfig(id) end,
								min = 1, max = 600, bigStep = 10,
							},
							offsetx = {
								order = 20,
								type = "range",
								name = L["Offset X"],
								get = function() return db.offsetX end,
								set = function(info, value) db.offsetX = value; self:UpdateConfig(id) end,
								min = -600, max = 600, bigStep = 10,
							},
							offsety = {
								order = 21,
								type = "range",
								name = L["Offset Y"],
								get = function() return db.offsetY end,
								set = function(info, value) db.offsetY = value; self:UpdateConfig(id) end,
								min = -600, max = 600, bigStep = 10,
							},
							frameLevel = {
								order = 23,
								type = "range",
								name = L["Frame level"],
								get = function() return db.frameLevel end,
								set = function(info, value) db.frameLevel = value; self:UpdateConfig(id) end,
								min = 1, max = 50, bigStep = 1,
							},
							anchor = {
								order = 31,
								type = "header",
								name = L["Anchor"],
							},
							anchorTo = {
								order = 31,
								type = "select",
								name = L["Anchor to"],
								values = module:AnchorList(id),
								get = function() return self:AnchorGet(db.anchor.to) end,
								set = function(info, value) self:AnchorSet(value, db); self:UpdateConfig(id) end,
							},
							point = {
								order = 42,
								type = "select",
								name = L["Anchor point"],
								values = module.relativeList,
								get = function() return db.anchor.point end,
								set = function(info, value) db.anchor.point = value; self:UpdateConfig(id) end,
							},
							relativePoint = {
								order = 43,
								type = "select",
								name = L["Anchor relative"],
								values = module.relativeList,
								get = function() return db.anchor.relative end,
								set = function(info, value) db.anchor.relative = value; self:UpdateConfig(id) end,
							},
						},
					},
					look = {
						type = "group",
						order = 20,
						name = L["Look & Feel"],
						childGroups = "tab",
						plugins = {},
						args = {
							desctexture = {
								order = 60,
								type = "description",
								name = L["Texture Desc1"],
							},
							texture = {
								order = 70,
								type = "select",
								name = L["Texture"],
								values = module:GetTextures(),
								get = function() return db.texture end,
								set = function(info, value) db.texture = value; self:UpdateConfig(id) end,
							},
							texturepath = {
								order = 71,
								type = "input",
								name = L["Texture path"],
								get = function() return mBar.media.mediaTable.statusbar[db.texture] or db.texture end,
								set = function(info, value) db.texture = value; self:UpdateConfig(id) end,
							},
							descrot = {
								order = 79,
								type = "description",
								name = L["Bar Rotation Desc1"],
							},
							rotation = {
								order = 80,
								type = "select",
								name = L["Bar Rotation"],
								get = function() return db.rotation end,
								set = function(info, value) db.rotation = value; self:UpdateConfig(id) end,
								values = self.rotation,
							},
							desctexturerot = {
								order = 90,
								type = "description",
								name = L["Texture Rotation Desc1"],
							},
							textureRot = {
								order = 91,
								type = "select",
								name = L["Texture Rotation"],
								get = function() return db.textureRot end,
								set = function(info, value) db.textureRot = value; self:UpdateConfig(id) end,
								values = self.textureRot,
							},
							desctextrot = {
								order = 95,
								type = "description",
								name = L["Texture Flip Desc1"],
							},
							textureFlip = {
								order = 96,
								type = "toggle",
								name = L["Texture Flip"],
								get = function() return db.textureFlip end,
								set = function(info, value) db.textureFlip = value; self:UpdateConfig(id) end,
							},
							textureMirror = {
								order = 97,
								type = "toggle",
								name = L["Texture Mirror"],
								get = function() return db.textureMirror end,
								set = function(info, value) db.textureMirror = value; self:UpdateConfig(id) end,
							},
						},
					},
				},
			}
			
			-- Module configs for the current bar
			for name, module in mHud:IterateModules() do
				if self.modConfig[name] and self.modConfig[name].bar and mHud.db.profile.module[name].active then
					local args = assert(loadstring(self.modConfig[name].bar)) ()
					local tab = args(db, id, self)
					self:InjectOptions(t['bar'..id].args, tab)
				end
			end
			
		end
	end
	
	if self.copy.from then
	
		t.bulk.args.descspace1 = { order = 19, type = "description", name = " " }
		t.bulk.args.desccopy1 = {
			order = 20,
			type = "description",
			name = L["Copy Select Bars"],
		}
		t.bulk.args.desccopy1line = { order = 21, type = "description", name = line }
		
		-- List bars
		self.copy.bars = { }
		for id,db in pairs(mHud.db.profile.bar) do
			t.bulk.args['bar'..id] = {
				order = 30+id,
				type = "toggle",
				name = L['Bar']..id..' '..db.unit..' '..db.type,
				get = function() return self.copy.bars[id] end,
				set = function(info, value) self.copy.bars[id] = value end,
				disabled = function() return (id == self.copy.from) end,
			}
		end

		t.bulk.args.descspace2 = { order = 49, type = "description", name = " " }
		t.bulk.args.desccopy2 = {
			order = 50,
			type = "description",
			name = L["Copy Attributes Bars"],
		}
		t.bulk.args.desccopy2line = { order = 51, type = "description", name = line }
		
		-- List atribs
		self.copy.atr = { }
		self.copy.order = 60
		self:AtribList(t)
		
		t.bulk.args.exec = {
			order = self.copy.order + 1,
			type = "execute",
			name = L["Copy"],
			func = function()
				self:CopyAtribList()
				for barId,barCopy in pairs(self.copy.bars) do
					if barCopy and barId ~= self.copy.from then
						mBar:ConfigChange(mBar:GetFrame(barId))
					end
				end
				self:WriteConfig()
			end,
		}
	end
	
	return t

end

function module:AtribList(t, db, pre)

	local send
	if not db then db = mHud.defaults.profile.bar['*'] end
	if not pre then pre = ""; send = "" else send = pre.." " end

	-- All the locals
	for k,v in pairs(db) do
		local id = send..k

		if type(v) ~= "table" then
			t.bulk.args['atrib'..id] = {
				order = self.copy.order,
				type = "toggle",
				name = id,
				get = function() return self.copy.atr[id] end,
				set = function(info, value) self.copy.atr[id] = value end,
			}
		end
		
		self.copy.order = self.copy.order + 1
		
	end
	
	-- All sub trees
	for k,v in pairs(db) do
		local id = send..k

		if type(v) == "table" and k ~= '*' then -- Skip sub structures, like texts
			t.bulk.args['atrib'..id..'space'] = {
				order = self.copy.order + 0,
				type = "description",
				name = " ",
			}
			t.bulk.args['atrib'..id..'header'] = {
				order = self.copy.order + 1,
				type = "description",
				name = k,
			}
			t.bulk.args['atrib'..id..'line'] = { order = self.copy.order + 2, type = "description", name = line }
			self.copy.order = self.copy.order + 10
			self:AtribList(t, db[k], id)
		end
		
		self.copy.order = self.copy.order + 1
		
	end
	
end


function module:CopyAtribList(db, pre, tbl)

	local send
	if not tbl then tbl = { } end
	if not db then db = mHud.defaults.profile.bar['*'] end
	if not pre then pre = ""; send = "" else send = pre.." " end
	
	-- All the locals
	for k,v in pairs(db) do
		local id = send..k
		tbl[# tbl + 1] = k
		if type(v) ~= "table" then
			if self.copy.atr[id] then
				for barId,barCopy in pairs(self.copy.bars) do
					if barCopy and barId ~= self.copy.from then
						--mHud:Debug("copy to: "..barId, true)
						local dbTreeTo = mHud.db.profile.bar[barId]
						local dbTreeFrom = mHud.db.profile.bar[self.copy.from]
						for _,key in ipairs(tbl) do
							--mHud:Debug("Key: "..key, true)
							if type(dbTreeFrom[key]) == "table" then
								dbTreeTo = dbTreeTo[key]
								dbTreeFrom = dbTreeFrom[key]
							else
								dbTreeTo[key] = dbTreeFrom[key]
								--mHud:Debug("COPY: "..key, true)
								break
							end
						end
					end
				end
			end
		end
		tbl[# tbl] = nil
		
	end
	
	-- All sub trees
	for k,v in pairs(db) do
		local id = send..k

		if type(v) == "table" and k ~= '*' then -- Skip sub structures, like texts
			self:CopyAtribList(db[k], id, tbl)
		end
		
	end
	
end


function module:SkinMenu()
	
	local t = { }
	
	
	local i = 10
	for name, skin in mHud.skin:IterateModules() do
		for scheme,_ in pairs(skin.profiles) do
			local keyName = name.."key"..scheme
			local active = ""
			
			if mHud.db.profile.skin == name and mHud.db.profile.scheme == scheme then
				 active = " (active)"
			end
			
			t[keyName..'header'] = {
				order = i,
				type = "description",
				name = name.." - "..scheme..active,
			}
			t[keyName..'line'] = { order = i + 1, type = "description", name = line }
			t[keyName..'desc'] = {
				order = i + 2,
				type = "description",
				name = skin.description[scheme] or L["No description"],
			}
			t[keyName..'exec'] = {
				order = i + 3,
				type = "execute",
				name = L["Skin Change To"].." "..name.." - "..scheme,
				func = function()
				
					module:SetEdit(false)
					mHud:OnDisable()
					
					mHud:Debug("Reset profile: "..mHud.db:GetCurrentProfile().." with "..(name or "nil").." - "..(scheme or "nil"), true)
					
					mHud.db:ResetProfile()
					mHud.db.profile.skin = name
					mHud.db.profile.scheme = scheme
					mHud.skin.reset = true
					
					mHud:OnEnable()
					module:WriteConfig()
				end,
				confirm = true,
				confirmText = L["Skin Confirm Change"],
				disabled = function() return (mHud.db.profile.skin == name and mHud.db.profile.scheme == scheme) end,
			}
			t[keyName..'space'] = { order = i + 9, type = "description", name = " " }
			i = i + 10
		end
	end
	
	return t
	
end


function module:ModuleMenu()
	
	local t = { }
	local i = 20
	local arg = function() return {} end
	
	-- List of modules loaded
	t['modules'] = {
		order = 1, 
		type = "description",
		name = L["Modules"]..":",
	}
	t['modulesline'] = { order = 2, type = "description", name = line }
	t['modulesnote1'] = {
		order = 200, 
		type = "description",
		name = L["Modules Note1"],
	}
	
	for name, module in mHud:IterateModules() do
	
		-- Add to list of modules loaded
		t['modules'..name] = {
			order = i*2, 
			type = "toggle",
			name = name,
			get = function() return mHud.db.profile.module[name].active end,
			set = function(info, value)
				mHud.db.profile.module[name].active = value
				if not value then
					mHud:DisableModule(name)
				else
					mHud:EnableModule(name)
				end
				self:ReloadmHud()
			end,
			disabled = function() return (name == "Configuration") end,
		}
		t['modules'..name..'space'] = { order = i*2+1, type = "description", name = " " }
		
		-- If this module has a global config
		if self.modConfig[name] and self.modConfig[name].module and mHud.db.profile.module[name].active then
			if name == "Background" then self:Debug(self.modConfig[name].module, true) end
			
			local args = assert(loadstring(self.modConfig[name].module)) ()
			t[name] = {
				type = "group",
				order = i,
				name = name,
				childGroups = "tab",
				args = args(name, self),
			}
		end
		i = i + 1

	end
	
	return t
	
end


function module:HandlerMenu()

	local t = { }
	local i = 20
	local arg = function() return {} end
	
	-- List of modules loaded
	t['modules'] = {
		order = 1, 
		type = "description",
		name = L["Handlers"]..":",
	}
	t['modulesline'] = { order = 2, type = "description", name = line }
	
	for name, module in mBar:IterateModules() do
	
		-- Add to list of modules loaded
		t['modules'..name] = {
			order = i, 
			type = "description",
			name = "* "..name,
		}
		
		-- If this module has a global config
		if self.modConfig[name] then
			local args = assert(loadstring(self.modConfig[name].module)) ()
			t[name] = {
				type = "group",
				order = i,
				name = name,
				childGroups = "tab",
				args = args(name),
			}
		end
		i = i + 1

	end
	
	return t
	
end


function module:ResetProfile()

	module:SetEdit(false)
	mHud:OnDisable()

	local skin = mHud.db.profile.skin
	local scheme = mHud.db.profile.scheme
	
	mHud:Debug("Reset profile: "..mHud.db:GetCurrentProfile(), true)
	
	mHud.db:ResetProfile()
	mHud.db.profile.skin = skin
	mHud.db.profile.scheme = scheme
	mHud.db:RegisterDefaults(mHud.defaults)
	
	mHud.skin.reset = true
	
	mHud:OnEnable()
	module:WriteConfig()
	
end


function module:SetProfile(info, value)
	mHud:Debug("Change to profile: "..value, true)
	mHud.skin.isLoaded = false
	module:SetEdit(false)
	mHud:OnDisable()
	mHud.db:SetProfile(value)
	mHud.db:RegisterDefaults(mHud.defaults)
	mHud.skin.profileName = value
	mHud:OnEnable()
	module:WriteConfig()
end


function module:ReloadmHud()
	mHud:OnDisable()
	mHud:OnEnable()
	self:WriteConfig()
end

function module:NewModuleConfig(name, modTbl, barTbl, bulkTbl)

	self.modConfig[name] = {
		module = modTbl,
		bar = barTbl,
		bulk = bulkTbl,
	}

end

function module:InjectOptions(opt, tab)

	if type(tab) ~= "table" then error("mHud:SaveToDb only exepts tables") end
	for k,v in pairs(tab) do
		if not opt[k] then
			opt[k] = v
		else
			for i,p in pairs(v) do
				opt[k].plugins[i] = p
			end
		end
	end
	
end


function module:EditOnUpdate()

	if self.editValues.last + 1 / mHud.db.profile.configuration.fps > GetTime() then return end
	
	self.editValues.last = GetTime()
	self.editValues.value = self.editValues.value + mHud.db.profile.configuration.step * self.editValues.dir
	
	if self.editValues.value < 0 then
		self.editValues.value = 0
		self.editValues.dir = 1
	elseif self.editValues.value > 1 then
		self.editValues.value = 1
		self.editValues.dir = -1
	end

	for _,f in pairs(mHud.bars) do
		if type(f.handler.RegisterEvents) == "function" then
			mBar:SetValue(f, self.editValues.value)
		end
	end

end


function module:EnterEditMode()
	
	mHud:UnregisterEvents()
	for _,f in pairs(mHud.bars) do
		if type(f.handler.UnregisterEvents) == "function" then
			f.handler:UnregisterEvents()
			f:Show()
			f:SetAlpha(1)
		end
	end
	
	self.event:SetScript("OnUpdate", function() self:EditOnUpdate() end)
	
end

function module:LeaveEditMode()
	
	mHud:RegisterEvents()
	for _,f in pairs(mHud.bars) do
		if type(f.handler.RegisterEvents) == "function" then
			f.handler:RegisterEvents()
			mBar:SetValue(f)
		end
	end
	
	-- Get check for existance for all units
	for unit, _ in pairs(mHud.units or { }) do
		mHud:UpdateBarsForUnit(unit)
	end
	
	self.event:SetScript("OnUpdate", nil)
	
end

local corners = {
	BOTTOM		= "TOP",
	BOTTOMLEFT	= "TOPRIGHT",
	LEFT		= "RIGHT",
	TOPLEFT		= "BOTTOMRIGHT",
	CENTER		= "CENTER",
}

function module:EnterDragMode()

	self.event:SetScript("OnUpdate", nil)
	
	for _,f in pairs(mHud.bars) do
		if not f.edit then f.edit = { } end
		if not f.edit.bg then f.edit.bg = f:CreateTexture(nil, "ARTWORK") end
		--f.edit.bg:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Background")
		--f.edit.bg:SetAlpha(0.3)
		f.edit.bg:SetTexture(0, 0, 0, 0.3)
		f.edit.bg:SetAllPoints(f)
		
		for z,c in pairs(corners) do
			f.edit[z] = CreateFrame("Frame", nil, f)
			self:DragCorner(f.edit[z], z, c)
			
			f.edit[c] = CreateFrame("Frame", nil, f)
			self:DragCorner(f.edit[c], c, z)
		end
		
		f:EnableMouse(true)
		f:SetScript("OnEnter", function() self:OnDragEnter(f) end)
		f:SetScript("OnLeave", function() self:OnDragLeave(f) end)
		
	end
	
	self:ShowCenterCorners()
	
end


function module:DragCorner(f, c, z)

	f:SetMovable(true)
	f:EnableMouse(true)
	f:SetScript("OnMouseDown", function() self:StartMoving(f) end)
	f:SetScript("OnMouseUp", function() self:StopMoving(f) end)
	f:SetHeight(10)
	f:SetWidth(10)
	
	if not f.bg then f.bg = f:CreateTexture(nil, "ARTWORK") end
	
	if c ~= "CENTER" then
		f.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollUp-Up")

		--SetTexCoord(ULx,ULy,LLx,LLy,URx,URy,LRx,LRy);


		
		if c == "TOP" then
			--f.bg:SetTexCoord( .25, .75, .25, .75)
			f.bg:SetTexCoord(	.25, .25,
								.25, .75,
								.75, .25,
								.75, .75)
			
		elseif c == "BOTTOM" then
			f.bg:SetTexCoord( .75, .25, .75, .25)
			
		elseif c == "RIGHT" then	
			f.bg:SetTexCoord(	.25, .75,
								.75, .75,
								.25, .25,
								.75, .25)
			
		elseif c == "LEFT" then	
			f.bg:SetTexCoord(	.75, .25,
								.25, .25,
								.75, .75,
								.25, .75)
			
		elseif c == "TOPLEFT" then
			f.bg:SetTexCoord(	.50, .25,
								.10, .50,
								.90, .50,
								.50, .90)
			
		elseif c == "TOPRIGHT" then
			f.bg:SetTexCoord(	.10, .50,
								.50, .90,
								.50, .25,
								.90, .50)
			
		elseif c == "BOTTOMLEFT" then
			f.bg:SetTexCoord(	.90, .50,
								.50, .25,
								.50, .90,
								.10, .50)
			
		elseif c == "BOTTOMRIGHT" then
			f.bg:SetTexCoord(	.50, .90,
								.90, .50,
								.10, .50,
								.50, .25)
			
		end

	else
		f.bg:SetTexture("Interface\\Icons\\Spell_Nature_RavenForm")
		f.bg:SetTexCoord(    0.0625,     0.0625,
						     0.0625, 1 - 0.0625,
						 1 - 0.0625,     0.0625,
						 1 - 0.0625, 1 - 0.0625)
	end
	

	f.bg:SetAllPoints(f)
	
	f.self = c
	f.opose = z
	
	f:SetScript("OnEnter", function() self:OnDragEnter(f:GetParent()) end)
	f:SetScript("OnLeave", function() self:OnDragLeave(f:GetParent()) end)
	
	
	
	f:Show()
	f.bg:Show()
	
end


function module:UpdateCorners(f)

	for z,c in pairs(corners) do
		f.edit[c]:ClearAllPoints()
		f.edit[z]:ClearAllPoints()
	end

	-- Only show corners if the size is >= 20x20
	if f:GetWidth() >= 20 and f:GetHeight() >= 20  then

		f.edit["TOPLEFT"]:SetPoint("TOPLEFT", "UIParent", "BOTTOMLEFT", f:GetLeft(), f:GetBottom() + f:GetHeight())
		f.edit["TOPRIGHT"]:SetPoint("TOPRIGHT", "UIParent", "BOTTOMLEFT", f:GetLeft() + f:GetWidth(), f:GetBottom() + f:GetHeight())

		f.edit["BOTTOMLEFT"]:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", f:GetLeft(), f:GetBottom())
		f.edit["BOTTOMRIGHT"]:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", f:GetLeft() + f:GetWidth(), f:GetBottom())
		
		f.edit["TOPLEFT"]:Show()
		f.edit["TOPRIGHT"]:Show()
		f.edit["BOTTOMLEFT"]:Show()
		f.edit["BOTTOMRIGHT"]:Show()
	else
		f.edit["TOPLEFT"]:Hide()
		f.edit["TOPRIGHT"]:Hide()
		f.edit["BOTTOMLEFT"]:Hide()
		f.edit["BOTTOMRIGHT"]:Hide()
	end
	
	f.edit["LEFT"]:SetPoint("LEFT", "UIParent", "BOTTOMLEFT", f:GetLeft(), f:GetBottom() + f:GetHeight()/2 )
	f.edit["RIGHT"]:SetPoint("RIGHT", "UIParent", "BOTTOMLEFT", f:GetLeft() + f:GetWidth(), f:GetBottom() + f:GetHeight()/2 )
	
	f.edit["TOP"]:SetPoint("TOP", "UIParent", "BOTTOMLEFT", f:GetLeft() + f:GetWidth()/2, f:GetBottom() + f:GetHeight())
	f.edit["BOTTOM"]:SetPoint("BOTTOM", "UIParent", "BOTTOMLEFT", f:GetLeft() + f:GetWidth()/2, f:GetBottom())
	
	f.edit["CENTER"]:SetPoint("CENTER", "UIParent", "BOTTOMLEFT", f:GetLeft() + f:GetWidth()/2, f:GetBottom() + f:GetHeight()/2 )
	
	f.edit["LEFT"]:Show()
	f.edit["RIGHT"]:Show()
	f.edit["TOP"]:Show()
	f.edit["BOTTOM"]:Show()
	f.edit["CENTER"]:Show()
	
end


function module:HideCorners(centerTo)
	-- Hide them all
	for _,frame in pairs(mHud.bars) do
		for z,c in pairs(corners) do
			if z ~= "CENTER" or centerTo then
				frame.edit[c]:Hide()
				frame.edit[z]:Hide()
			end
		end
	end	
end

function module:ShowCornersFor(f)
	for z,c in pairs(corners) do
		f.edit[c]:Show()
		f.edit[z]:Show()
		
		if z == "CENTER" then
			f.edit[c]:SetFrameLevel(101)
			f.edit[z]:SetFrameLevel(101)
		else
			f.edit[c]:SetFrameLevel(f:GetFrameLevel()+1)
			f.edit[z]:SetFrameLevel(f:GetFrameLevel()+1)
		end
	end
	self:UpdateCorners(f)
end

function module:ShowCorners()
	for _,f in pairs(mHud.bars) do
		self:ShowCornersFor(f)
	end
end

function module:ShowCenterCorners()
	for _,f in pairs(mHud.bars) do
		f.edit["CENTER"]:Show()
		f.edit["CENTER"]:SetFrameLevel(101)
		f.edit["CENTER"]:SetPoint("CENTER", "UIParent", "BOTTOMLEFT", f:GetLeft() + f:GetWidth()/2, f:GetBottom() + f:GetHeight()/2 )
	end
end


function module:StartMoving(f)

	local p = f:GetParent()

	if not p.startPos then p.startPos = { } end
	if not p.endPos then p.endPos = { } end
	
	p.startPos.left = p:GetLeft()
	p.startPos.bottom = p:GetBottom()
	p.startPos.width = p:GetWidth()
	p.startPos.height = p:GetHeight()
	
	p:ClearAllPoints()
	
	p:SetPoint(f.opose, p.edit[f.opose], f.opose)
	p:SetPoint(f.self, f, f.self)
	
	self:HideCorners(true)
	
	if p.background then
		p.background:SetVertexColor(1, 1, 1, 1)
		p.bar:SetAlpha(0)
	end
	
	if p.frame then p.frame:SetAlpha(0) end
	
	self.event:SetScript("OnUpdate", function() self:DragOnUpdate(f) end)
	f:StartMoving()

end


function module:DragOnUpdate(f)

	-- Exit in case of sticky
	if not IsMouseButtonDown() then self:StopMoving(f) end

end


function module:StopMoving(f)

	f:StopMovingOrSizing()
	self.event:SetScript("OnUpdate", nil)
	
	local p = f:GetParent()
	
	p.endPos.left = p:GetLeft()
	p.endPos.bottom = p:GetBottom()
	p.endPos.width = p:GetRight() - p:GetLeft()
	p.endPos.height = p:GetTop() - p:GetBottom()
	
	if f.self ~= "CENTER" then
		
		if p.endPos.width < 0 then p.endPos.width = 1 end
		if p.endPos.height < 0 then p.endPos.height = 1 end
		
		if p.db.anchor.point == "CENTER" then
		
			local divX = 2
			local divY = 2
			
			if f.self == "TOPRIGHT" or f.self == "RIGHT" or f.self == "BOTTOMRIGHT" then
				divX = -2
			end
			
			if f.self == "TOP" or f.self == "TOPLEFT" or f.self == "TOPRIGHT" then
				divY = -2
			end
			
			p.db.offsetX = p.db.offsetX + (p.db.width - p.endPos.width)/divX
			p.db.offsetY = p.db.offsetY + (p.db.height - p.endPos.height)/divY
			
		else
		
			local divX = 2
			local divY = 2
			
			if f.self == "TOPRIGHT" or f.self == "RIGHT" or f.self == "BOTTOMRIGHT" then
				divX = -2
			end
			
			if f.self == "TOP" or f.self == "TOPLEFT" or f.self == "TOPRIGHT" then
				divY = -2
			end
			
			p.db.offsetX = p.db.offsetX + (p.db.width - p.endPos.width)/divX
			p.db.offsetY = p.db.offsetY + (p.db.height - p.endPos.height)/divY
			
		end
		
		p.db.width = p.endPos.width
		p.db.height = p.endPos.height
		
		mHud:Debug(p.endPos.width)
		--mHud:Debug(f:GetParent():GetLeft().."x"..f:GetParent():GetRight().." Diff: "..(f:GetParent():GetLeft()), true)
	else
		
		p.db.offsetX = p.db.offsetX + (p.endPos.left - p.startPos.left)
		p.db.offsetY = p.db.offsetY + (p.endPos.bottom - p.startPos.bottom)
		
	end
	
	mBar:ConfigChange(p)
	p:Show()

	-- Show all
	self:ShowCornersFor(p)
	self:ShowCenterCorners()
	
end


function module:LeaveDragMode()

	for _,f in pairs(mHud.bars) do
		if f.edit then
			if f.edit.bg then f.edit.bg:Hide() end
			
			for _,c in pairs(corners) do
				if f.edit[c] then f.edit[c]:Hide() end
			end
			
			f.edit = nil
		end
		
		f:EnableMouse(false)
		f:SetScript("OnEnter", nil)
		f:SetScript("OnLeave", nil)
	end
	
	self.event:SetScript("OnUpdate", function() self:EditOnUpdate() end)

end

	
function module:OnDragEnter(f)

	-- Show BG with full strength
	f.edit.bg:SetAlpha(1)
	
	-- Remember old level, if it isn't allready the top level
	if f:GetFrameLevel() ~= 100 then
		f.oldLevel = f:GetFrameLevel()
	end
	
	-- Set to top level
	f:SetFrameLevel(100)
	self:ShowCornersFor(f)
	
	f.edit.bg:SetVertexColor(1, 0, 0)
end


function module:OnDragLeave(f)

	-- Dim background again
	f.edit.bg:SetAlpha(0.3)
	
	-- Back to the old level
	f:SetFrameLevel(f.oldLevel)
	
	-- Hide all the corners
	self:HideCorners()
	
end






