local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
if not mHud or not mHud:GetModule("Configuration", true) then return end

local barConfig = [[ return function(db, id, self)
		local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
		local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")
		local module = mHud:GetModule("Configuration")
		local DogTag = LibStub("LibDogTag-2.0")
		
		local childGrp = "tree"
		--if (# db.text > 4) then childGrp = "tree" end
		
		local t = {
			text = {
				type = "group",
				name = "Text",
				order = 100,
				childGroups = childGrp,
				args = {
					new = {
						type = "group",
						name = "New",
						order = 200,
						childGroups = "tab",
						args = {
							desc1 = {
								order = 10,
								type = "description",
								name = "Add a new text assosiated to this bar.",
							},
							tag = {
								order = 1,
								type = "execute",
								name = "Add text",
								func = function()
									local textId = # db.text + 1
									db.text[textId].anchor.to = id
									db.text[textId].anchor.point = "TOP"
									db.text[textId].anchor.relative = "BOTTOM"
									mHud:GetModule("DogTag"):OnBarCreate(nil, mBar:GetFrame(id))
									module:WriteConfig();
								end,
							},
						},
					},
				},
			},
		}
		
		for textId,textDB in pairs(db.text) do
			t.text.args['text'..textId] = {
				type = "group",
				name = textId..": "..textDB.tag,
				order = textId,
				childGroups = "tab",
				args = {
					general = {
						type = "group",
						order = 20,
						name = "General",
						childGroups = "tab",
						plugins = {},
						args = {
							tag = {
								order = 1,
								type = "input",
								name = "DogTag",
								get = function() return textDB.tag end,
								set = function(info, value) textDB.tag = value; module:UpdateConfig(id); module:WriteConfig(); end,
							},
							color = {
								order = 1,
								type = "color",
								name = "Font Color",
								get = function() return textDB.color.r, textDB.color.g, textDB.color.b, textDB.color.a end,
								set = function(info, r, g, b, a)
									textDB.color.r=r
									textDB.color.g=g
									textDB.color.b=b
									textDB.color.a=a
									module:UpdateConfig(id);
								end,
							},
							font = {
								order = 8,
								type = "select",
								name = "Font Face",
								values = mBar.media:List("font"),
								get = function()
									for k,v in ipairs(mBar.media:List("font")) do
										if v == textDB.font then return k end
									end
								end,
								set = function(info, value)
									for k,v in ipairs(mBar.media:List("font")) do
										if k == value then
											textDB.font = v
											module:UpdateConfig(id);
										end
									end
									return
								end,
							},
							space1 = {
								order = 9,
								name = " ",
								type = "description",
							},
							x = {
								order = 10,
								type = "range",
								name = "X Position",
								get = function() return textDB.offsetX end,
								set = function(info, value) textDB.offsetX = value; module:UpdateConfig(id); end,
								min = -600, max = 600, bigStep = 10,
							},
							y = {
								order = 11,
								type = "range",
								name = "Y Position",
								get = function() return textDB.offsetY end,
								set = function(info, value) textDB.offsetY = value; module:UpdateConfig(id); end,
								min = -600, max = 600, bigStep = 10,
							},
							--sizedesc = {
							--	order = 19,
							--	type = "description",
							--	name = "Size isn't working prop. Reload UI after setting this fixes the size problem.",
							--},
							size = {
								order = 20,
								type = "range",
								name = "Size",
								get = function() return textDB.size end,
								set = function(info, value) textDB.size = value; module:UpdateConfig(id); end,
								min = 1, max = 30, step = 1,
							},
						},
					},
					anchor = {
						type = "group",
						order = 30,
						name = "Anchor",
						childGroups = "tab",
						plugins = {},
						args = {
							anchorTo = {
								order = 10,
								type = "select",
								name = "Anchor To",
								values = module:AnchorList(),
								get = function() return self:AnchorGet(textDB.anchor.to) end,
								set = function(info, value) self:AnchorSet(value, textDB); self:UpdateConfig(id) end,
							},
							point = {
								order = 20,
								type = "select",
								name = "Point",
								values = mHud:GetModule("Configuration").relativeList,
								get = function() return textDB.anchor.point end,
								set = function(info, value) textDB.anchor.point = value; module:UpdateConfig(id)
								end,
							},
							relativePoint = {
								order = 30,
								type = "select",
								name = "Relative Point",
								values = mHud:GetModule("Configuration").relativeList,
								get = function() return textDB.anchor.relative end,
								set = function(info, value) textDB.anchor.relative = value; module:UpdateConfig(id)
								end,
							},
						},
					},
					shadow = {
						type = "group",
						order = 40,
						name = "Shadow",
						childGroups = "tab",
						plugins = {},
						args = {
							shadow = {
								order = 30,
								type = "color",
								name = "Shadow Color",
								get = function() return textDB.shadow.r, textDB.shadow.g, textDB.shadow.b, textDB.shadow.a end,
								set = function(info, r, g, b, a)
									textDB.shadow.r=r
									textDB.shadow.g=g
									textDB.shadow.b=b
									textDB.shadow.a=a
									module:UpdateConfig(id)
								end,
							},
							space1 = {
								order = 31,
								name = " ",
								type = "description",
							},
							shadowx = {
								order = 40,
								type = "range",
								name = "Shadow X offset",
								get = function() return textDB.shadow.offsetX end,
								set = function(info, value) textDB.shadow.offsetX = value; module:UpdateConfig(id); end,
								min = -10, max = 10, bigStep = 1,
							},
							shadowy = {
								order = 41,
								type = "range",
								name = "Shadow Y offset",
								get = function() return textDB.shadow.offsetY end,
								set = function(info, value) textDB.shadow.offsetY = value; module:UpdateConfig(id); end,
								min = -10, max = 10, bigStep = 1,
							},
						},
					},
					copy = {
						type = "group",
						order = 50,
						name = "Copy",
						childGroups = "tab",
						plugins = {},
						args = {
							copy = {
								order = 10,
								type = "description",
								name = "Copy from other texts",
							},
						},
					},
					delete = {
						type = "group",
						order = 60,
						name = "Delete",
						childGroups = "tab",
						plugins = {},
						args = {
							delete = {
								order = 10,
								type = "execute",
								name = "Remove this textbox",
								func = function()
									DogTag:RemoveFontString(mBar:GetFrame(id).text[textId])
									mBar:GetFrame(id).text[textId]:Hide()
									db.text[textId] = nil
									module:WriteConfig()
								end,
								confirm = true,
								confirmText = "This wil compleatly remove the text. Would you like to continue?",
							},
						},
					},
				},
			}
			
			if mHud.mAlphaTemplateConfig then
				local short = t.text.args['text'..textId].args.general
				short.plugins.alpha = { }
				short.plugins.alpha.alphanormal = {
					type = "select",
					name = "Alpha - Normal",
					order = 6,
					values = mHud.mAlphaTemplateConfig:List(2, "Normal"),
					get = function() return textDB.gradientAlpha end,
					set = function(info, value) textDB.gradientAlpha = value; self:UpdateConfig(id) end,
				}
				short.plugins.alpha.alphacombat = {
					type = "select",
					name = "Alpha - Combat",
					order = 7,
					values = mHud.mAlphaTemplateConfig:List(2, "Combat", true),
					get = function() return textDB.gradientAlphaCombat or "nil" end,
					set = function(info, value) textDB.gradientAlphaCombat = value; if value == "nil" then textDB.gradientAlphaCombat = nil end; self:UpdateConfig(id) end,
				}
			end
		end
		
		return t
		end]]
local bulkConfig = nil
local moduleConfig = nil --[[ return function(name) return {
			desc1 = {
				type = "description",
				name = "This is the "..name.." configuration",
			}
		} end]]
mHud:GetModule("Configuration"):NewModuleConfig("DogTag", moduleConfig, barConfig, bulkConfig)
