local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")
local DogTag = LibStub("LibDogTag-2.0")
local module = mHud:NewModule("DogTag")
module.text = { } -- Storage of all fontstrings

mHud.defaults.profile.bar['*'].text = {
	['*'] = {

		tag = "[CurHP]",
		size = 14,
		
		offsetX = 0,
		offsetY = 0,
		
		font = "Arial Narrow",
		gradientAlpha = "Default",
		gradientAlphaCombat = "Bright",
		
		color = {
			r = 1,
			g = 1,
			b = 1,
			a = 1
		},
		
		shadow = {
			r = 0,
			g = 0,
			b = 0,
			a = 1,
			offsetX = -1,
			offsetY = 0,
		},
		
		anchor = {
			point = "CENTER",
			to = "UIParent",
			relative = "CENTER",
		},
	}
}


function module:Enable()

	self:Debug("Enable")
	
	self:RegisterMessage("MBAR_CREATED", "OnBarCreate")
	self:RegisterMessage("MBAR_REMOVE", "OnBarRemove")
	self:RegisterMessage("MBAR_CONFIG_UPDATE", "OnBarConfigChange")
	self:RegisterMessage("MBAR_ANCHOR_SET", "OnAnchor")
	
	self:RegisterMessage("MBAR_PAINT", "OnPaint")
	
end


function module:OnBarCreate(_, f)

	if f.db.text then
		
		if not f.text then f.text = { } end
		
		for id,db in pairs(f.db.text) do
			self:Debug("Creating text")
			if not f.text[id] then f.text[id] = f:CreateFontString(nil, "OVERLAY") end -- f:GetName().."Font"..id
			f.text[id].db = db
		end
		
	end

end


function module:OnBarConfigChange(_, f)

	if f.text then
		for id,t in pairs(f.text) do
			if t and t.db then -- After removing a text bar, this might be nil

				t:Show()
				--t:SetTextHeight(t.db.size)
				t:SetFont(mBar.media:Fetch("font", t.db.font), t.db.size)
				--
				
				--if mHud:GetModule("GradientAlpha", true) and mHud:GetModule("GradientAlpha"):IsEnabled() then
				--	t:SetVertexColor( t.db.color.r, t.db.color.g, t.db.color.b )
				--else
					t:SetVertexColor( t.db.color.r, t.db.color.g, t.db.color.b, f.bar:GetAlpha() )
				--end

				t:SetShadowColor( t.db.shadow.r, t.db.shadow.g, t.db.shadow.b, t.db.shadow.a )									
				t:SetShadowOffset( t.db.shadow.offsetX, t.db.shadow.offsetY )
				
				DogTag:AddFontString(t, f, f.db.unit, t.db.tag or '[CurHP]')
				
				self:Debug("Updated position etc.")
				
			end
		end
	end
	
end

function module:OnAnchor(_, f)
	if f.text then
		for id,t in pairs(f.text) do
			if t and t.db then -- After removing a text bar, this might be nil
				t:ClearAllPoints()
				t:SetPoint(
					-- Point
					t.db.anchor.point or "CENTER",
					-- To
					(type(t.db.anchor.to) == "number" and mBar:GetFrame(t.db.anchor.to))
					or t.db.anchor.to
					or "UIParent",
					-- Relative
					t.db.anchor.relative or "CENTER",
					-- X
					t.db.offsetX,
					-- Y
					t.db.offsetY)
			end
		end
	end
end

function module:OnBarRemove(_, f)

	if f.text then
		for id,t in pairs(f.text) do
			if t and t.db then -- After removing a text bar, this might be nil
				self:Debug("Remove text: "..f.id.."-"..id)
				t:Hide()
				DogTag:RemoveFontString(t)
				--table.remove(module.text, t.tId)
			end
		end
	end
	
end

function module:OnPaint(_, f)
	
	if f.db.text then
		for id,_ in pairs(f.db.text) do
			DogTag:UpdateFontString(f.text[id])
		end
		-- Gradient
	end
	
end

