local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
if not mHud or not mHud:GetModule("Configuration", true) then return end
local barConfig = [[ return function(db, id, self)
		local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")

		local t =  {
			falloff = {
				type = "group",
				order = 100,
				name = "Falloff / Barfade",
				childGroups = "tab",
	 			plugins = {},
				args = {
					toggle = {
						type = "toggle",
						name = "Use Falloff",
						order = 2,
						get = function() return db.falloff.active end,
						set = function(info, value) db.falloff.active = value; self:UpdateConfig(id) end,
					},
					space = {
						type = "description",
						name = " ",
						order = 3,
					},
					desc1 = {
						type = "description",
						name = "FPS is the number of times the falloff elements should move each second. A combined FPS of 30 and 30 step, will leave the falloffs visible for 1 second.\n\nNumber of elements is the maximum falloffs for the bar, before it will reuse the old once. You should enable edit mode and change the values to get an idea of what it does.",
						order = 4,
					},
					fps = {
						type = "range",
						name = "FPS",
						order = 5,
						get = function() return db.falloff.fps end,
						set = function(info, value) db.falloff.fps = value end,
						disabled = function() return not db.falloff.active end,
						min = 1, max = 90, bigStep = 5,
					},
					elements = {
						type = "range",
						name = "Number of elements",
						order = 6,
						get = function() return db.falloff.elements end,
						set = function(info, value) db.falloff.elements = value; self:UpdateConfig(id) end,
						disabled = function() return not db.falloff.active end,
						min = 1, max = 30, bigStep = 1,
					},
				},
			},
		}
		
		------------
		-- Incoming
		------------
		
		for dir,name in pairs({incoming = "Incoming", outgoing = "Outgoing"}) do
			
			t.falloff.args[dir] = {
				type = "group",
				order = 5,
				name = name,
				childGroups = "tab",
				plugins = {},
				args = {},
			}
			
			t.falloff.args[dir].args.toggle = {
				type = "toggle",
				name = "Enable",
				order = 11,
				get = function() return db.falloff[dir].active end,
				set = function(info, value) db.falloff[dir].active = value; self:UpdateConfig(id) end,
				disabled = function() return not db.falloff.active end,
			}
			
			t.falloff.args[dir].args.invert = {
				type = "toggle",
				name = "Move towards the bar?",
				order = 12,
				get = function() return db.falloff[dir].invert end,
				set = function(info, value) db.falloff[dir].invert = value end,
				disabled = function() return not db.falloff.active end,
			}
			
			t.falloff.args[dir].args.steps = {
				type = "range",
				name = "Total steps",
				order = 20,
				get = function() return db.falloff[dir].steps end,
				set = function(info, value) db.falloff[dir].steps = value end,
				disabled = function() return not (db.falloff.active and db.falloff[dir].active) end,
				min = 1, max = 200, bigStep = 5,
			}
			
			t.falloff.args[dir].args.stepsspace = {
				type = "description",
				name = " ",
				order = 21,
			}
			
			t.falloff.args[dir].args.dx = {
				type = "range",
				name = "X Step length",
				order = 30,
				get = function() return db.falloff[dir].dx end,
				set = function(info, value) db.falloff[dir].dx = value end,
				disabled = function() return not (db.falloff.active and db.falloff[dir].active) end,
				min = -10, max = 10, bigStep = 0.01,
			}
			
			t.falloff.args[dir].args.dy = {
				type = "range",
				name = "Y Step length",
				order = 31,
				get = function() return db.falloff[dir].dy end,
				set = function(info, value) db.falloff[dir].dy = value end,
				disabled = function() return not (db.falloff.active and db.falloff[dir].active) end,
				min = -10, max = 10, bigStep = 0.01,
			}
				
			if mHud:GetModule("GradientColor", true) then
				t.falloff.args[dir].args.colornormal = {
					type = "select",
					name = "Colors - Normal",
					order = 50,
					values = mHud.mColorTemplateConfig:List(2, "Normal"),
					get = function() return db.falloff[dir].gradientColor end,
					set = function(info, value) db.falloff[dir].gradientColor = value; self:UpdateConfig(id) end,
					disabled = function() return not (db.falloff.active and db.falloff[dir].active) end,
				}
				t.falloff.args[dir].args.colorcombat = {
					type = "select",
					name = "Colors - Combat",
					order = 51,
					values = mHud.mColorTemplateConfig:List(2, "Combat", true),
					get = function() return db.falloff[dir].gradientColorCombat or "nil" end,
					set = function(info, value) db.falloff[dir].gradientColorCombat = value; if value == "nil" then db.falloff[dir].gradientColorCombat = nil end; self:UpdateConfig(id) end,
					disabled = function() return not (db.falloff.active and db.falloff[dir].active) end,
				}
			else
				t.falloff.args[dir].args.color = {
					type = "color",
					name = "Colors - Combat",
					order = 51,
					get = function() return db.falloff[dir].color.r, db.falloff[dir].color.g, db.falloff[dir].color.b end,
					set = function(info, r, g, b) db.falloff[dir].color.r = r; db.falloff[dir].color.g = g; db.falloff[dir].color.b = b; end,
					disabled = function() return not (db.falloff.active and db.falloff[dir].active) end,
					hasAlpha = true,
				}
			end
			
			if mHud:GetModule("GradientAlpha", true) then
				t.falloff.args[dir].args.alphanormal = {
					type = "select",
					name = "Alpha - Normal",
					order = 55,
					values = mHud.mAlphaTemplateConfig:List(2, "Normal"),
					get = function() return db.falloff[dir].gradientAlpha end,
					set = function(info, value) db.falloff[dir].gradientAlpha = value; self:UpdateConfig(id) end,
					disabled = function() return not (db.falloff.active and db.falloff[dir].active) end,
				}
				t.falloff.args[dir].args.alphacombat = {
					type = "select",
					name = "Alpha - Combat",
					order = 56,
					values = mHud.mAlphaTemplateConfig:List(2, "Combat", true),
					get = function() return db.falloff[dir].gradientAlphaCombat or "nil" end,
					set = function(info, value) db.falloff[dir].gradientAlphaCombat = value; if value == "nil" then db.falloff[dir].gradientAlphaCombat = nil end; self:UpdateConfig(id) end,
					disabled = function() return not (db.falloff.active and db.falloff[dir].active) end,
				}
			end
			
		end
		
		return t
		end]]
local bulkConfig = nil
local moduleConfig = nil
mHud:GetModule("Configuration"):NewModuleConfig("Falloff", moduleConfig, barConfig, bulkConfig)
