local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")
local module = mHud:NewModule("Falloff")
local e = CreateFrame("Frame", nil)
module.updateList = { }
module.holdTo = GetTime() + 1
module.alpha = {  }
module.color = {  }

mHud.defaults.profile.bar['*'].falloff = {
	active = false,
	
	incoming = {
		active = false,
		color = { r = 0, g = 1, b = 0, a = 1.00 }, -- Color if gradient color isn't active
		gradientColor = "FalloffHealing",
		gradientAlpha = "Falloff",
		invert = true,
		dx = 0.5,
		dy = 0.0,		
		steps = 60,
	},
	
	outgoing = {
		active = false,
		color = { r = 1, g = 0, b = 0, a = 1.00 }, -- Color if gradient color isn't active
		gradientColor = "FalloffDamage",
		gradientAlpha = "Falloff",
		invert = false,
		dx = 0.5,
		dy = 0.0,		
		steps = 60,
	},
		
	elements = 5,
	fps = 30,
}


module.colorTemplates = {
	Normal = {
		FalloffDamage = {
			[1] = { v = 1.0, c = { r = 1.00, g = 0.00, b = 0.00 }},
		},
		FalloffHealing = {
			[1] = { v = 1.0, c = { r = 0.00, g = 1.00, b = 0.00 }},
		},
	},
}

module.alphaTemplates = {
	Normal = {
		Falloff = {
			[1] = { v = 1.00, c = { a = 0.50, b = "BLEND" }},
			[2] = { v = 0.70, c = { a = 1.00, b = "BLEND" }},
			[3] = { v = 0.00, c = { a = 1.00, b = "BLEND" }},
		},
	},
}

function module:OnEnable() self:RegisterEvents() end
function module:OnDisable() self:UnegisterEvents() end

function module:RegisterEvents()

	self:RegisterMessage("MBAR_PAINT", "Paint")
	self:RegisterMessage("MBAR_CONFIG_UPDATE", "BarConfigChange")
	
end


function module:UnegisterEvents()

	self:UnregisterMessage("MBAR_PAINT")
	self:UnregisterMessage("MBAR_CONFIG_UPDATE")
	
end

function module:Paint(_, f)

	if self.holdTo > GetTime() or not f.falloff then return end
	
	self.ff = f.falloff[ f.falloff.el ]
	
	if math.abs(f.oldValue - f.value) > 0.001 then 
		if (f.oldValue or 0) < f.value then
			self.ff.db = f.db.falloff.incoming
			self.ff.low = f.oldValue or 0
			self.ff.high = f.value
		else
			self.ff.db = f.db.falloff.outgoing
			self.ff.high = f.oldValue
			self.ff.low = f.value
		end
	else -- Fix the tiny once
		if (f.oldValue or 0) < f.value then
			self.ff.db = f.db.falloff.incoming
			self.ff.low = f.oldValue or 0
			self.ff.high = (f.oldValue or 0) + 0.001
		else
			self.ff.db = f.db.falloff.outgoing
			self.ff.high = f.value + 0.001
			self.ff.low = f.value
		end
	end
	
	if not self.ff.db.active then return end
	
	mBar:PaintTexture(self.ff, self.ff.low, self.ff.high)
	
	-- Color
	if mHud:GetModule("GradientColor", true) and mHud:GetModule("GradientColor"):IsEnabled() then
		self.color = mHud:GetModule("GradientColor"):GetProfile(f, self.ff.db.gradientColor, self.ff.db.gradientColorCombat, f.value)
		self.ff:SetVertexColor(self.color.r, self.color.g, self.color.b)
	else
		self.ff:SetVertexColor(ff.db.color.r, ff.db.color.g, ff.db.color.b, ff.db.color.a)
	end
	
	-- Alpha
	if mHud:GetModule("GradientAlpha", true) and mHud:GetModule("GradientAlpha"):IsEnabled() then
		self.alpha = mHud:GetModule("GradientAlpha"):GetProfile(f, self.ff.db.gradientAlpha, self.ff.db.gradientAlphaCombat, f.value)
		self.ff.startAlpha = self.alpha.a
		self.ff:SetAlpha( self.ff.startAlpha )
		self.ff:SetBlendMode(self.alpha.b)
	end
	
	if self.ff.db.invert then
		self.ff.x = - self.ff.db.steps * self.ff.db.dx
		self.ff.y = - self.ff.db.steps * self.ff.db.dy
	else
		self.ff.x = 0
		self.ff.y = 0
	end
	
	self.ff.die = GetTime()
	self.ff.nextRun = 0
	self.ff.step = 0
	
	self.ff:Show()
	self:Update(f, self.ff)
	if # self.updateList == 0 then e:SetScript("OnUpdate", function() self:UpdateAll() end) end
	tinsert(self.updateList, {f, self.ff})
	
	f.falloff.el = f.falloff.el + 1
	if not f.falloff[ f.falloff.el ] then f.falloff.el = 1 end
	
end

function module:UpdateAll()

	if # self.updateList == 0 then e:SetScript("OnUpdate", nil); return end	
	
	for id,v in pairs(self.updateList) do
		self:Update(v[1], v[2], id)
	end
	
end

function module:Update(f, ff, id)
	
	if ff.nextRun > GetTime() then return end
	ff.nextRun = GetTime() + 1/f.db.falloff.fps
	
	if ff.step > ff.db.steps then
		--ff:SetScript("OnUpdate", nil)
		tremove(self.updateList, id)
		ff:Hide()
		return
	end
	
	mBar:SetSubValue(ff, ff.low, ff.high, ff.step * ff.db.dx + ff.x, ff.step * ff.db.dy + ff.y)
	
	if ff.step > ff.db.steps*2/3 then		
		-- ( 1 - (( 3i / s )   -2)) * start alpha
		ff:SetAlpha( (1 - (3 * ff.step / ff.db.steps) + 2) * ff.startAlpha )
	end
	
	ff.step = ff.step + 1
	
end

function module:BarConfigChange(_, f)

	self.holdTo = GetTime() + 0.2

	if f.db.falloff.active then
		if not f.falloff then f.falloff = { } end
		local i
		for i = 1, 30 do
			if i > f.db.falloff.elements then
				f.falloff[i] = nil
			else
				if not f.falloff[i] then
					f.falloff[i] = f:CreateTexture(nil, "OVERLAY")
				end
				f.falloff[i]:SetWidth(f:GetWidth())
				f.falloff[i]:SetHeight(f:GetHeight())
				f.falloff[i]:SetTexture(f.bar:GetTexture())
				f.falloff[i]:Hide()
			end
		end
		if not f.falloff.el or not f.falloff[ f.falloff.el ] then f.falloff.el = 1 end
	else
		if f.falloff then f.falloff = nil end
	end
end
