local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
local module = mHud:NewModule("GradientAlpha")
mHud.mTemplate:Embed(module)
--module.defTemplate = "Default"
--module.defTemplateCombat = "Default"

local alphaModes = {
	DISABLE = true,
	BLEND = true,
	ALPHAKEY = true,
	ADD = true,
	MOD = true,
}

mHud.defaults.profile.bar['*'].gradientAlpha = "AUTO_POWER"
mHud.defaults.profile.bar['*'].gradientAlphaCombat = "AUTO_POWER"

module.defaultTemplates = {
	Normal = {
		Default = {
			[1] = { v = 1.00, c = { a = 0.05, b = "BLEND" }},
			[2] = { v = 0.98, c = { a = 0.20, b = "BLEND" }},
			[3] = { v = 0.70, c = { a = 0.20, b = "BLEND" }},
			[4] = { v = 0.01, c = { a = 1.00, b = "BLEND" }},
			[5] = { v = 0.00, c = { a = 0.10, b = "BLEND" }},
		},
		Bright = {
			[1] = { v = 1.00, c = { a = 1.00, b = "BLEND" }},
		},
		Hidden = {
			[1] = { v = 1.00, c = { a = 0.00, b = "BLEND" }},
		},
	},
	Combat = {
		Default = {
			[1] = { v = 1.00, c = { a = 0.10, b = "BLEND" }},
			[2] = { v = 0.90, c = { a = 0.50, b = "BLEND" }},
			[3] = { v = 0.60, c = { a = 1.00, b = "BLEND" }},
			[4] = { v = 0.01, c = { a = 1.00, b = "BLEND" }},
			[5] = { v = 0.00, c = { a = 0.10, b = "BLEND" }},
		},
	},
}


function module:OnEnable()
	
	self:LoadModuleTemplates("alphaTemplates")
	self:RegisterEvents()	
	self.customTemplates = mHud.db.profile.templates.alpha
	
end


function module:RegisterEvents()

	self:RegisterMessage("MBAR_PAINT", "SetAlpha")
	self:RegisterMessage("MBAR_CREATED", "BarCreated")
	self:RegisterMessage("MBAR_CONFIG_UPDATE", "BarConfigChange")
	
end


function module:OnDisable()

	self:UnegisterEvents()

end


function module:UnegisterEvents()

	self:UnregisterMessage("MBAR_PAINT", "SetAlpha")
	self:UnregisterMessage("MBAR_CREATED", "BarCreated")
	self:UnregisterMessage("MBAR_CONFIG_UPDATE", "BarConfigChange")
	
end

function module:BarCreated(_, f)
	
	self:Debug("BarCreated")
	if not f.alpha then f.alpha = { a = 1, b = "BLEND" } end
	
end


function module:BarConfigChange(_, f)
	
	self:Debug("BarConfigChange")
	f.type = string.lower(f.db.type)
	
end


function module:SetAlpha(_, f)

	-- Bar
	self:GetProfile(f, f.db.gradientAlpha, f.db.gradientAlphaCombat, f.value)
	f.bar:SetAlpha(self.rTab.a)
	if alphaModes[self.rTab.b] then
		f.bar:SetBlendMode(self.rTab.b)
	end
	
	-- Text
	if f.text then
		for _,text in pairs(f.text) do
			self:GetProfile(f, text.db.gradientAlpha, text.db.gradientAlphaCombat, f.value)
			text:SetAlpha(self.rTab.a)
		end
	end
	
	-- Background
	if f.background then
		self:GetProfile(f, f.db.background.gradientAlpha, f.db.background.gradientAlphaCombat, f.value)
		f.background:SetAlpha(self.rTab.a)
		if alphaModes[self.rTab.b] then
			f.background:SetBlendMode(self.rTab.b)
		end
	end
	
	-- Frame
	if f.frame then
		self:GetProfile(f, f.db.frame.gradientAlpha, f.db.frame.gradientAlphaCombat, f.value)
		f.frame:SetAlpha(self.rTab.a)
		if alphaModes[self.rTab.b] then
			f.frame:SetBlendMode(self.rTab.b)
		end
	end
	
end
