local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")
local module = mHud:NewModule("GradientColor")
mHud.mTemplate:Embed(module)
--module.defTemplate = "Default"
--module.defTemplateCombat = "Default"

mHud.defaults.profile.bar['*'].gradientColor = "AUTO_POWER"
mHud.defaults.profile.bar['*'].gradientColorCombat = nil -- same as normal

module.defaultTemplates = {
	Normal = {
		Default = {
			[1] = { v = 1.0, c = { r = 0.00, g = 1.00, b = 0.00 }},
			[2] = { v = 0.5, c = { r = 1.00, g = 1.00, b = 1.00 }},
			[3] = { v = 0.2, c = { r = 1.00, g = 0.00, b = 1.00 }},
			[4] = { v = 0.0, c = { r = 0.00, g = 0.00, b = 0.00 }},
		},
		-- Friendly and hostile
		Friendly = {
			[1] = { v = 1.0, c = { r = 0.00, g = 1.00, b = 0.00 }},
		},
		Neutral = {
			[1] = { v = 1.0, c = { r = 1.00, g = 1.00, b = 0.00 }},
		},
		Hostile = {
			[1] = { v = 1.0, c = { r = 1.00, g = 0.00, b = 0.00 }},
		},
		-- Other
		White = {
			[1] = { v = 1.0, c = { r = 1.00, g = 1.00, b = 1.00 }},
		},
		Black = {
			[1] = { v = 1.0, c = { r = 0.00, g = 0.00, b = 0.00 }},
		},
		Trapped = {
			[1] = { v = 1.0, c = { r = 0.50, g = 0.50, b = 0.50 }},
		},
	},
	Combat = {
		Trapped = {
			[1] = { v = 1.0, c = { r = 0.50, g = 0.50, b = 0.50 }},
		},
	},
}

-- Copy class colors
for k,v in pairs(RAID_CLASS_COLORS) do
	module.defaultTemplates.Normal[string.upper(k)] = { [1] = { v = 1.0, c = { r = v.r, g = v.g, b = v.b }}}
end


function module:OnEnable()

	self:LoadModuleTemplates("colorTemplates")
	self:RegisterEvents()
	module.oldSetColor = mBar.SetColor
	mBar.SetColor = function() end
	
	self.customTemplates = mHud.db.profile.templates.color
	
end


function module:RegisterEvents()

	self:RegisterMessage("MBAR_PAINT", "SetColor")
	self:RegisterMessage("MBAR_CREATED", "BarCreated")
	self:RegisterMessage("MBAR_CONFIG_UPDATE", "BarConfigChange")
	
end


function module:OnDisable()

	self:UnegisterEvents()
	mBar.SetColor = module.oldSetColor
	
end


function module:UnegisterEvents()

	self:UnregisterMessage("MBAR_PAINT", "SetColor")
	self:UnregisterMessage("MBAR_CREATED", "BarCreated")
	self:UnregisterMessage("MBAR_CONFIG_UPDATE", "BarConfigChange")
	
end


function module:BarCreated(_, f)
	
	self:Debug("BarCreated")
	if not f.color then f.color = { r = 1, g = 1, b = 1 } end
	
end


function module:BarConfigChange(_, f)
	
	self:Debug("BarConfigChange")
	f.type = string.lower(f.db.type)
	
end


function module:SetColor(_, f)

	self:GetProfile(f, f.db.gradientColor, f.db.gradientColorCombat, f.value, f.color)
	f.bar:SetVertexColor(self.rTab.r, self.rTab.g, self.rTab.b)
	
	-- Background
	if f.background then
		self:GetProfile(f, f.db.background.gradientColor, f.db.background.gradientColorCombat, f.value, f.color)
		f.background:SetVertexColor(self.rTab.r, self.rTab.g, self.rTab.b)
	end
		
	-- Frame
	if f.frame then
		self:GetProfile(f, f.db.frame.gradientColor, f.db.frame.gradientColorCombat, f.value, f.color)
		f.frame:SetVertexColor(self.rTab.r, self.rTab.g, self.rTab.b)
	end
	
end
