if LibStub("AceAddon-3.0"):GetAddon("mBar", true) then return end

local debug = false

local mBar = LibStub("AceAddon-3.0"):NewAddon("mBar")

LibStub("AceEvent-3.0"):Embed(mBar)
mBar.media = LibStub("LibSharedMedia-2.0")
mBar.frames = { }

function mBar:OnEnable()
	
	self:RegisterMessage("MBAR_PAINT", function(_, f)
		self:PaintTexture(f.bar, 0, f.value)
		self:SetColor(f)
	end)
	self:RegisterMessage("MBAR_UPDATE", function(_, f) self:SetValue(f) end)
	
end

function mBar:SetColor(f) f.bar:SetVertexColor(f.handler:GetColor(f)) end


function mBar:CreateBar(id, db)

	--local f = self:GetFrame(id)	-- Get old, or create new frame
	--if not f then f = CreateFrame("Frame",  self:GetFrameName(id)) end
	local f =  CreateFrame("Frame", nil, UIParent)
	
	f.id = id
	f.db = db
	f.handler = self:GetModule(f.db.type or "None", true)
	if not f.handler then return nil end -- No handler => No bar
	f:Hide() -- Default hidden
	
	-- Make the handler aware of this bar
	f.handler:NewBar(f)
	
	-- Get old, or create new texture
	if not f.bar then f.bar = f:CreateTexture(nil, "ARTWORK") end
	f.bar:Show()
	
	self.frames[id] = f
	
	-- Set parameters
	self:SendMessage("MBAR_CREATED", f)
	self:ConfigChange(f)
	
	return f
	
end


function mBar:ConfigChange(f)
	
	-- Frame
	f:SetHeight(f.db.height or 10)
	f:SetWidth(f.db.width or 100)
	f:ClearAllPoints()
	self:Anchor(f)
	
	f:SetFrameLevel(f.db.frameLevel or 1)
	
	-- Update handler
	f.handler = self:GetModule(f.db.type or "None", true)
	
	-- Bar
	f.bar:SetTexture(mBar.media.mediaTable.statusbar[f.db.texture] or f.db.texture or "Interface\\TargetingFrame\\UI-StatusBar")
	
	f.bar:ClearAllPoints()
	if f.db.rotation ~= 0 and f.db.rotation
						  then f.bar:SetPoint("RIGHT", f, "RIGHT") end
	if f.db.rotation ~= 1 then f.bar:SetPoint("BOTTOM", f, "BOTTOM") end
	if f.db.rotation ~= 2 then f.bar:SetPoint("LEFT", f, "LEFT") end
	if f.db.rotation ~= 3 then f.bar:SetPoint("TOP", f, "TOP") end
	
	self:SendMessage("MBAR_CONFIG_UPDATE", f)
	self:SetValue(f)
	
end


function mBar:Anchor(f)

	f:SetPoint(
		-- Point
		f.db.anchor.point or "CENTER",
		-- To
		(type(f.db.anchor.to) == "number" and self:GetFrame(f.db.anchor.to))
		or f.db.anchor.to
		or "UIParent",
		-- Relative
		f.db.anchor.relative or "CENTER",
		-- X
		f.db.offsetX,
		-- Y
		f.db.offsetY)
	self:SendMessage("MBAR_ANCHOR_SET", f)
	
end


function mBar:GetValue(f) return f.handler:GetValue(f.db.unit) end


function mBar:SetValue(f, value)

	if not value then value = f.handler:GetValue(f) end
	if value < 0.001 then value = 0.001 end
	if value > 1 then value = 1 end
	
	f.oldValue = f.value
	f.value = value
	
	if f.db.rotation == 0 or f.db.rotation == 2 then
		f.bar:SetWidth(f:GetWidth() * value)
	else
		f.bar:SetHeight(f:GetHeight() * value)
	end
	
	self:SendMessage("MBAR_PAINT", f)
	self:SendMessage("MBAR_PAINT_OVERLAY", f)
	
end

--[[
	Poor readability, but higher performance and minimum garbage
--]]
function mBar:SetSubValue(f, low, high, dx, dy)

	local p = f:GetParent()
	
	if low < 0 then low = 0 end
	if high > 1 then high = 1 end
	
	if p.db.rotation == 0 or p.db.rotation == 2 then
		f:SetWidth((high - low) * p:GetWidth())
		if p.db.rotation == 0 then
			f:SetPoint("BOTTOMLEFT", p, "BOTTOMLEFT", low * p:GetWidth() + (dx or 0), 0 + (dy or 0));
		else -- 2
			f:SetPoint("BOTTOMLEFT", p, "BOTTOMLEFT", p:GetWidth() * (1 - low - (high - low)) + (dx or 0), 0 + (dy or 0));
		end
	else
		f:SetHeight((high - low) * p:GetHeight())
		if p.db.rotation == 3 then
			f:SetPoint("BOTTOMLEFT", p, "BOTTOMLEFT", 0 + (dx or 0), low * p:GetHeight() + (dy or 0));
		else -- 1
			f:SetPoint("BOTTOMLEFT", p, "BOTTOMLEFT", 0 + (dx or 0), p:GetHeight() * (1 - low - (high - low)) + (dy or 0));
		end
	end
	
	self:SendMessage("MBAR_SUB_PAINT", f)
	
end

--local onze = false

function mBar:PaintTexture(tex, fr, to)

	local f = tex:GetParent()

	-- Work the magic ; )
	-- If we're rotationg the bar and the texture, they counter eachouther
	
	--if not f.db.textureRot and onze == false then
		--LibStub("AceAddon-3.0"):GetAddon("mHud"):PrintTable(LibStub("AceAddon-3.0"):GetAddon("mHud").defaults.profile.bar['*'], 0, false)
		--LibStub("AceAddon-3.0"):GetAddon("mHud"):PrintTable(f.db, 0, false)
		--onze = true
	--end
	
	if not f.db.rotation then return end
	
	tex.rot = f.db.rotation - f.db.textureRot
	if tex.rot < 0 then tex.rot = tex.rot + 4 end
	
	if tex.rot == 0 then
		tex.ulT = {fr, 0}
		tex.dlT = {fr, 1}
		tex.urT = {to, 0}
		tex.drT = {to, 1}
	elseif tex.rot == 1 then
		tex.ulT = {0, fr}
		tex.dlT = {0, to}
		tex.urT = {1, fr}
		tex.drT = {1, to}
	elseif tex.rot == 2 then
		tex.ulT = {1-to, 0}
		tex.dlT = {1-to, 1}
		tex.urT = {1-fr, 0}
		tex.drT = {1-fr, 1}
	elseif tex.rot == 3 then
		tex.ulT = {0, 1-to}
		tex.dlT = {0, 1-fr}
		tex.urT = {1, 1-to}
		tex.drT = {1, 1-fr}
	end
	
	if f.db.textureRot == 0 then
		tex.ulF = tex.ulT
		tex.dlF = tex.dlT
		tex.urF = tex.urT
		tex.drF = tex.drT
	elseif f.db.textureRot == 1 then
		tex.ulF = tex.dlT
		tex.dlF = tex.drT
		tex.urF = tex.ulT
		tex.drF = tex.urT
	elseif f.db.textureRot == 2 then
		tex.ulF = tex.drT
		tex.dlF = tex.urT
		tex.urF = tex.dlT
		tex.drF = tex.ulT
	else --if f.db.textureRot == 3 then
		tex.ulF = tex.urT
		tex.dlF = tex.ulT
		tex.urF = tex.drT
		tex.drF = tex.dlT
	end
	
	if f.db.textureMirror == true then
		-- flip X
		tex.ulF[1] = 1 - tex.ulF[1]
		tex.dlF[1] = 1 - tex.dlF[1]
		tex.urF[1] = 1 - tex.urF[1]
		tex.drF[1] = 1 - tex.drF[1]
	end
	
	if f.db.textureFlip == true then
		-- Flip Y
		tex.ulF[2] = 1 - tex.ulF[2]
		tex.dlF[2] = 1 - tex.dlF[2]
		tex.urF[2] = 1 - tex.urF[2]
		tex.drF[2] = 1 - tex.drF[2]
	end

	if not tex.ulF then -- returns nil for some reason sometimes when unit is target ?
		return
	end
	
	tex:SetTexCoord(tex.ulF[1], tex.ulF[2], tex.dlF[1], tex.dlF[2], tex.urF[1], tex.urF[2], tex.drF[1], tex.drF[2])
	
end


--function mBar:Hide(f) f:Hide() end
function mBar:RemoveBar(f)
	self:Debug("Remove: "..f.id)

	f:Hide()
	f.handler:RemoveBar(f)
	self:SendMessage("MBAR_REMOVE", f)
end

function mBar:GetFrame(id) return self.frames[id] end
--function mBar:GetFrameName(id) return "mHudBar" .. id end

function mBar:Debug(msg, force, module)
	if debug or force then
		if not module then
			module = "core"
		else
			local _, start = string.find(module, '_') -- Skip mBar_
			module = string.sub(module, start + 1)
		end
		DEFAULT_CHAT_FRAME:AddMessage("|cFFAAFF00mBar:|cFFFF5D51"..module..":|cFFFFFFFF"..msg)
	end
end

