local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")
--[[ Module prototypehandler --]]

local handler = { }
handler.debug = false -- Will enable debugging on all modules

function handler:OnEnable()
	self:Debug("OnEnable")
	if not self.event then self.event = CreateFrame("Frame") end
	self.bars = { }
	self.event:SetScript("OnEvent", function() self:OnEvent() end)
	self:RegisterEvents()
end

function handler:RegisterEvents()
	self:Debug("RegisterEvents")
	if self.events then
		for _, e in pairs(self.events) do self.event:RegisterEvent(e) end
	end
end

function handler:OnDisable()
	self:Debug("OnDisable"); self:UnregisterEvents()
	for _, f in pairs(self.bars) do self:RemoveBar(f) end
end

function handler:UnregisterEvents()
	self:Debug("UnregisterEvents")
	if self.events then
		for _, e in pairs(self.events) do self.event:UnregisterEvent(e) end
	end
end

function handler:OnEvent()
	self:Debug("OnEvent: "..event)
	if not arg1 then arg1 = "player" end
	if self.bars[arg1] then
		local value = self:GetUnitValue(arg1)
		for id, frame in pairs(self.bars[arg1]) do
			self:Debug("Update frame: "..frame.id)
			mBar:SetValue(frame, value)
		end
	end
end


function handler:NewBar(f)
	self:Debug("NewBar: "..f.id)
	if not self.bars[f.db.unit] then self.bars[f.db.unit] = {} end
	table.insert(self.bars[f.db.unit], f)
end


function handler:RemoveBar(f)
	self:Debug("RemoveBar: "..f.id)
	for id, frame in pairs(self.bars[f.db.unit] or { }) do
		if frame.id == f.id then table.remove(self.bars[f.db.unit], id) end
	end
end

function handler:GetColor(f) return 1, 1, 1 end
function handler:GetValue(f) return self:GetUnitValue(f.db.unit) end
function handler:GetUnitValue(unit) return 0 end
function handler:GetMax(f) return self:GetUnitMax(f.db.unit) end
function handler:GetUnitMax(unit) return 0 end
function handler:Debug(msg, force) mBar:Debug(msg, force or self.debug, self.name) end

mBar:SetDefaultModuleState(true)
mBar:SetDefaultModulePrototype(handler)