local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")

if mHud.mTemplate then return end

mHud.mTemplate = { }
local mTemplate = mHud.mTemplate

mTemplate.defaultTemplates = { }
mTemplate.customTemplates = { }
mTemplate.defTemplate = "Default"
mTemplate.defTemplateCombat = "Default"
mTemplate.inCombat = false
mTemplate.dynamicTemplates = {
	AUTO_POWER = "<Type/Power>",
	AUTO_CLASS = "<Class>",
	AUTO_HOSTILE = "<Hostile>",
}

mTemplate.rTab = {
	v = 1,
	a = 1,
	r = 1,
	g = 1,
	b = 1,
}

-- Storagespace for custom templates
mHud.defaults.profile.templates = {
	['*'] = { -- Template type
		Normal = {
			['*'] = { -- Template name
				['*'] = {
					v = -1,
					c = { }, -- Template content
				},
			},
		},
		Combat = {
			['*'] = { -- Template name
				['*'] = {
					v = -1,
					c = { }, -- Template content
				},
			},
		},
	},
}

function mTemplate:SetDefaults(tbl)

	if type(tbl) ~= "table" then error("mTemplate:SetDefaults only accepts tables") end
	self.defaultTemplates = tbl
	
end

function mTemplate:GetTemplateVars(name, combat, value)
	--self:Debug("TPL: "..name..(combat and "COMBAT" or "IDLE"), true)
	local tpl
	if combat then
		--if name == "ManaTicker" then self:Debug("JA: "..name, true) end
		tpl = (self.customTemplates.Combat[name][1].v ~= -1 and self.customTemplates.Combat[name]) or self.defaultTemplates.Combat[name] or (self.customTemplates.Combat[self.defTemplateCombat][1].v ~= -1 and self.customTemplates.Combat[self.defTemplateCombat]) or self.defaultTemplates.Combat[self.defTemplateCombat]
	end
	if not combat or not tpl then
		--self:Debug(type(self.customTemplates.Normal), true)
		--if not self.customTemplates.Normal then return end
		tpl = (self.customTemplates.Normal and self.customTemplates.Normal[name] and self.customTemplates.Normal[name][1] and self.customTemplates.Normal[name][1].v ~= -1 and self.customTemplates.Normal[name]) or self.defaultTemplates.Normal[name] or (self.customTemplates.Normal[self.defTemplateCombat][1].v ~= -1 and self.customTemplates.Normal[self.defTemplateCombat]) or self.defaultTemplates.Normal[self.defTemplate]
	end
	if type(tpl) ~= "table" then error(("Template don't exist %q"):format(name)) end
	
	-- The tickers sometime sends values over 1, this causes errors
	if value > 1 then value = 1 end
	

	-- Allow single templates
	if # tpl == 1 then
		for k,v in pairs(tpl[1].c) do
			self.rTab[k] = v
		end
		return self.rTab
	end
	
	for k,v in ipairs(tpl) do
		if value > v.v then
			if not tpl[k-1] then
				self.rTab = v.c
			else
				-- Gradient all numbers, leave the rest
				local w = (tpl[k-1].v - value) / (tpl[k-1].v - v.v)
				for i,t in pairs(v.c) do
					if type(tpl[k-1].c[i]) == "number" then
						self.rTab[i] =
						tpl[k-1].c[i] * (1-w) 
						+ v.c[i] * w
					elseif type(tpl[k-1].c[i]) == "string" then
						self.rTab[i] = t
					else
						-- Returns here on custom alpha templates, du to the lack of b = "ADD" key
						if i == "b" then self.rTab[i] = "BLEND" end
						--self:Debug(name.." for value "..value.." key "..k.."-"..i.." )> nil   tpl[k-1].c[i]", true)
					end
				end
			end
			return self.rTab
		end
	end
	return self.rTab
	-- Sometimes end up here..
	--error(("No match %q"):format(name))

end

function mTemplate:TemplateExists( name, state )
	return self.customTemplates[state][name][1].v ~= -1 or self.defaultTemplates[state][name] ~= nil
end

function mTemplate:GetProfile(f, normal, combat, value)
	if (combat or self:TemplateExists( normal, "Combat" )) and (UnitAffectingCombat(f.db.unit) or mTemplate.inCombat) then
		if combat then normal = combat end
		combat = true
	else
		combat = false
	end
	
	if normal == "AUTO_POWER" then

		if UnitIsTapped(f.db.unit) and not UnitIsTappedByPlayer(f.db.unit) then
			normal = "Trapped"
		else
			if f.db.type == "Power" then
				if (UnitPowerType(f.db.unit) == 0) then
					normal = "Mana"
				elseif (UnitPowerType(f.db.unit) == 1) then	
					normal = "Rage"
				elseif (UnitPowerType(f.db.unit) == 2) then	
					normal = "Focus"
				elseif (UnitPowerType(f.db.unit) == 3) then	
					normal = "Energy"
				end
			elseif f.db.type == "DruidMana" then
				normal = "Mana"
			else
				normal = f.db.type
			end
		end
		
	elseif normal == "AUTO_CLASS" then
	
		normal = select(2, UnitClass(f.db.unit))

	elseif normal == "AUTO_HOSTILE" then
		
		if UnitIsTapped(f.db.unit) and not UnitIsTappedByPlayer(f.db.unit) then
			normal = "Trapped"
		else
			local hostile = UnitReaction(f.db.unit, "player");
			if hostile then
				if hostile < 4 then
					normal = "Hostile"
				elseif hostile > 4 then
					normal = "Friendly"
				else
					normal = "Neutral"
				end
			end
		end
			
	end

	if not normal then normal = "__fale__" end
	self:Debug("GetProfile() result: "..(normal))
	
	return self:GetTemplateVars(normal, combat, value)
	
end

-- Register defaults from other modules
function mTemplate:LoadModuleTemplates(key, silent)
	if silent == nil then silent = true end
	for _, module in mBar:IterateModules() do self:LoadModuleTemplatesFrom(module[key], silent) end
	for _, module in mHud:IterateModules() do self:LoadModuleTemplatesFrom(module[key], silent) end
	for _, module in mHud.skin:IterateModules() do self:LoadModuleTemplatesFrom(module[key], true) end
	
end

function mTemplate:LoadModuleTemplatesFrom(tbl, silent)
	silent = false
	if tbl and type(tbl) == "table" then
		for cat,t in pairs(tbl) do
			if not self.defaultTemplates[cat] then
				if not silent then error("Category does not exist, try 'Normal' or 'Combat'.") end
			else		
				for k,v in pairs(t) do
					if not self.defaultTemplates[cat][k] or v.overWrite == true then
						self.defaultTemplates[cat][k] = v
					else
						if not silent then mHud:Debug(cat.." - "..k.." exists, ignoring the new one.") end
					end
				end
			end
		end
	end
end

function mTemplate:Embed(inTo)
	for k,v in pairs(self) do
		inTo[k] = v
	end
end
