local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")

if mHud.mTemplateConfig then return end

local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")

mHud.mTemplateConfig = { }
local mTemplateConfig = mHud.mTemplateConfig

mTemplateConfig.temp = { }
mTemplateConfig.editName = nil
mTemplateConfig.editBranch = nil
mTemplateConfig.modId = ""
mTemplateConfig.modName = ""
mTemplateConfig.newTpl = { }


function mTemplateConfig:EditTemplate(t)

	if not self.editName or not self.editBranch then return end
	
	local tpl
	self.temp = { } -- reset 

	if self.editName ~= "[New]" then
		if mHud.db.profile.templates[self.modId][self.editBranch][self.editName][1].v ~= -1 then
			tpl = mHud.db.profile.templates[self.modId][self.editBranch][self.editName]
		else
			tpl = mHud:GetModule(self.modName, true).defaultTemplates[self.editBranch][self.editName]
		end
	else
		tpl = {[1] = self.newTpl}
	end

	if not tpl then
		mHud:Debug("tpl is empty", true, "mTemplateComfig")
		return
	end
	
	for id,v in ipairs(tpl) do

		if not self.temp[id] then self.temp[id] = { } end
		if not self.temp[id].c then self.temp[id].c = { } end
	
		self.temp[id].v = v.v
	
		if self.editName ~= "[New]" then
			t['value'..id] = {
				order = id*10,
				type = "range",
				name = "Value",
				get = function() return self.temp[id].v end,
				set = function(info, value) self.temp[id].v = value end,
				min = 0, max = 1, bigStep = 0.01,
			}
			self:EditTemplateConfig(t, id, v.c)
		end
	end
	
	if self.editName then
		
		t['save'] = {
			order = 220,
			type = "execute",
			name = "Save",
			func = function() self:Save() end,
		}
		
		if self.editName ~= "[New]" then
			t['add'] = {
				order = 210,
				type = "execute",
				name = "Add Value",
				func = function() self:NewValue() end,
			}
		
			if mHud:GetModule(self.modName, true).defaultTemplates[self.editBranch][self.editName] then
				if mHud.db.profile.templates[self.modId][self.editBranch][self.editName][1].v ~= -1 then
					t['revert'] = {
						order = 230,
						type = "execute",
						name = "Revert to default",
						func = function()
							mHud.db.profile.templates[self.modId][self.editBranch][self.editName] = nil
							for id, f in pairs(mHud.bars) do mBar:SetValue(f) end -- Repaint all bars
							mHud:GetModule("Configuration"):WriteConfig()
						end,
					}
				end

			else
				t['delete'] = {
					order = 230,
					type = "execute",
					name = "Delete",
					func = function()
						mHud.db.profile.templates[self.modId][self.editBranch][self.editName] = nil
						self.editName = nil
						mHud:GetModule("Configuration"):WriteConfig()
					end,
				}
			end
		else
			self.editName = "Enter a name"
			t['name'] = {
				order = 210,
				type = "input",
				name = "Template name",
				get = function() return self.editName end,
				set = function(info, value) self.editName = value end,
			}
			t['namenote'] = {
				order = 220,
				type = "description",
				name = "!PS! Press <Enter> or <OK> on the text box before you <Save>, or the name won't be registered!",
			}
		end
	end
	
	t['desctemplatespace0'] = {
		order = 247,
		type = "description",
		name = " ",
	}
	
	t['desctemplatehead'] = {
		order = 248,
		type = "description",
		name = "How To Use",
	}
	t['desctemplateline'] = {
		order = 249,
		type = "description",
		name = "----------------------------",
	}
	t['desctemplate1'] = {
		order = 250,
		type = "description",
		name = "The form above allows you to modify default templates, or create your own.",
	}
	t['desctemplate1space'] = { order = 251, type = "description", name = " " }
	
	t['desctemplate2'] = {
		order = 260,
		type = "description",
		name = "To the left you have the value of the bar, ranging from 0 to 1 (full).",
	}
	t['desctemplate2space'] = { order = 261, type = "description", name = " " }
	
	t['desctemplate3'] = {
		order = 270,
		type = "description",
		name = "On the right is the data assosiated with this value.",
	}
	t['desctemplate3space'] = { order = 271, type = "description", name = " " }
	
	t['desctemplate4'] = {
		order = 280,
		type = "description",
		name = "If there is only one value, the bar will allways have the one setting spesified.",
	}
	t['desctemplate4space'] = { order = 281, type = "description", name = " " }
	
	t['desctemplate5'] = {
		order = 290,
		type = "description",
		name = "When there is more than one value, the addon will make a smooth transition between each level.",
	}
	t['desctemplate5space'] = { order = 291, type = "description", name = " " }
	
	t['desctemplate6'] = {
		order = 300,
		type = "description",
		name = "Teknical info:",
	}
	t['desctemplate6line'] = {
		order = 301,
		type = "description",
		name = "----------------------------",
	}
	t['desctemplate7'] = {
		order = 310,
		type = "description",
		name = "Value: 0.001 - the bar is 0, but to avoid graphical errors, it's 0.001",
	}

end

function mTemplateConfig:EditTemplateConfig(t, id, v) end

function mTemplateConfig:NewValue()
	local k = #self.temp + 1
	self.temp[k] = { }
	self.temp[k].v = self.newTpl.v
	self.temp[k].c = { }
	
	-- Copy values
	for i,v in pairs(self.newTpl.c) do self.temp[k].c[i] = v end

	self:Save()
	mHud:GetModule("Configuration"):WriteConfig()
end

function mTemplateConfig:Save()
	-- Order, highest first
	local k
	local t = { }
	while #self.temp > 0 do
		k = self:FindHighestKey(self.temp)
		table.insert(t, self.temp[k])
		table.remove(self.temp, k)
	end
	
	t[1].v = 1 -- First is allways 1
	if #t > 1 then t[#t].v = 0 end -- Last is allways 0
	
	mHud:SaveToDb(mHud.db.profile.templates[self.modId][self.editBranch], t, self.editName)
	-- Update all bars
	for id, f in pairs(mHud.bars) do mBar:SetValue(f) end -- Repaint all bars
	
	-- Create new config
	mHud:GetModule("Configuration"):WriteConfig()
end

function mTemplateConfig:FindHighestKey(t)
	local high = 0
	local highKey = nil
	for k,v in pairs(t) do
		if not highKey or v.v > high then
			high  = v.v
			highKey = k
		end
	end
	return highKey
end


function mTemplateConfig:List(dynamic, branch, clear)
	
	local t = {}
	
	if not mHud:GetModule(self.modName, true) then return t end
	
	if clear then t['nil'] = '<Same as Normal>' end
	if dynamic == 1 then
		t['[New]'] = '[New]'
	elseif dynamic == 2 then
		for k,v in pairs(mHud.mTemplate.dynamicTemplates) do
			t[k] = v
		end
	end

	for k,v in pairs(mHud.db.profile.templates[self.modId][branch] or { }) do
		if v[1].v ~= -1 then
			if not mHud:GetModule(self.modName, true).defaultTemplates[branch][k] then
				t[k] = k.." - User defined"
			else
				t[k] = k.." - Customized"
			end
		end
	end

	for k,v in pairs(mHud:GetModule(self.modName, true) and mHud:GetModule(self.modName, true).defaultTemplates[branch] or { }) do
		if not t[k] then
			t[k] = k
		end
	end
	
	return t
	
end


function mTemplateConfig:Embed(inTo)
	for k,v in pairs(self) do
		inTo[k] = v
	end
end
