local mHud = LibStub("AceAddon-3.0"):GetAddon("mHud")
local mBar = LibStub("AceAddon-3.0"):GetAddon("mBar")

if mHud.mTicker then return end

mHud.mTicker = { }
local mTicker = mHud.mTicker

mTicker.rTab = { }
mTicker.frames = { }
mTicker.lastRun = 0
mTicker.lastTick = 0
mTicker.active = true
mTicker.runing = false

mHud.defaults.profile.ticker = { 
	fps = 60,
	size = 1.5,
	loop = 10,
	color = { r = 1, g = 1, b = 1, a = 0.8 },
}

function mTicker:AddFrame(f)
	self:UpdateTickerConfig(f)
	for _, frame in pairs(self.frames) do if f.id == frame.id then return end end
	table.insert(self.frames, f)
end

function mTicker:RemoveFrame(f)
	for id, frame in pairs(self.frames) do
		if f.id == frame.id then
			table.remove(self.frames, id)
			return
		end
	end
end

function mTicker:UpdateTickerConfig(f)

	if not self.event then self.event = CreateFrame("Frame") end
	if not f.ticker then f.ticker = f:CreateTexture(nil, "OVERLAY") end
	f.ticker:SetHeight(f:GetHeight())
	f.ticker:SetWidth(f:GetWidth())
	f.ticker:SetTexture(f.bar:GetTexture())
	f.ticker:SetVertexColor(mHud.db.profile.ticker.color.r, mHud.db.profile.ticker.color.g, mHud.db.profile.ticker.color.b, mHud.db.profile.ticker.color.a)
	
end

function mTicker:StartTicker(endIn)
	
	self.endTime = endIn + GetTime()
	self.startTime = GetTime()
	for _,f in pairs(self.frames) do f.ticker:Show() end -- Show all tickers
	self.event:SetScript("OnUpdate", function() self:OnUpdate() end)
	self.runing = true
	--self:Debug("START: "..endIn, true)

end

function mTicker:GetPos() return 1 - ((self.endTime - GetTime()) / (self.endTime - self.startTime)) end

function mTicker:OnUpdate()

	if self.lastRun + 1 / mHud.db.profile.ticker.fps > GetTime() then return end
	self.lastRun = GetTime()
	
	self.pos = self:GetPos()

	for _,f in pairs(self.frames) do
		if f.rotation == 0 or f.rotation == 2 then
			self.low = self.pos - mHud.db.profile.ticker.size / f:GetWidth()
			self.high = self.pos + mHud.db.profile.ticker.size / f:GetWidth()
		else
			self.low = self.pos - mHud.db.profile.ticker.size / f:GetHeight()
			self.high = self.pos + mHud.db.profile.ticker.size / f:GetHeight()
		end

		if self.low < 0 then self.low = 0 end
		if self.high > 1 then self.high = 1 end

		mBar:SetSubValue(f.ticker, self.low, self.high)
		mBar:PaintTexture(f.ticker, self.low, self.high)

		self:OnTickerUpdate(f)
	end
	
	if self.endTime < GetTime() or self.active ~= true then
		self.event:SetScript("OnUpdate", nil)
		for _,f in pairs(self.frames) do f.ticker:Hide() end
		self.runing = false
		self:OnKillTicker()
	end
	
end

function mTicker:OnTickerUpdate(f) end

function mTicker:OnKillTicker() end

function mTicker:Embed(inTo)
	for k,v in pairs(self) do
		inTo[k] = v
	end
end
